//============================================================================
//FILE: CoreTypes.h
//
// TYPE: C++
// DESCRIPTION: définition des types "moteur" et des variables globales
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CORETYPES_H
#define _CORETYPES_H

// SYSTEM INCLUDES
//
//#include <deque.h>     // librairie STL dans /opt/gnu/lib/g++-include

//#ifdef _SO_POST42_
//  using namespace std;
//  #include <iostream>
//  #include <cstdlib>
//  #include <cstring>
//#else 
//  #include <iostream.h>
//  #include <stdlib.h>
//  #include <string.h>
//#endif

  using namespace std;
#include <stdio.h>
#include <iostream>
#include <cstdlib>
#include <cstring>

#include "CoreValues.h"  // pour MINDOUBLE  CB 22092005

// PROJECT INCLUDES
//
#include "Tableau.h"
#include "Liste.h"

class Entity;
class DescriptedEntity;
class BasicEntity;
class Simulation;
class Process;
class ContinuousProcess;
class DiscreteProcess; 
class Event;
class MetaClass;
class EntitySpec;
class DescValueSpec;
class DescIntValueSpec;
class DescFloatValueSpec;
class DescDoubleValueSpec;
class DescStringValueSpec;
class DescEntityValueSpec;
class Descriptor;  // pour Argument.h
class ConstantDescriptor;
class IntConstantDescriptor;
class FloatConstantDescriptor;
class DoubleConstantDescriptor;
class EntityConstantDescriptor;
class EntityTabConstantDescriptor;
class StringConstantDescriptor;
class VariableDescriptor;
class IntVariableDescriptor;
class FloatVariableDescriptor;
class DoubleVariableDescriptor;
class EntityVariableDescriptor;
class EntityTabVariableDescriptor;
class StringVariableDescriptor;
class StructureMonitor;
class DescValueMonitor;
class Parameter;
class File;
class OutputFile;
class Argument;
class Method;
class EntityMethod;
class ProcessMethod;
class EventMethod;
class MonitorMethod;
class EntitySpecMethod;
class SequentialDataFile;
class Date; // + JPR 080822

class RNG; // + JPR 090730

class ClockValueItem;
class ChangeStructureItem;
class StructEventData;
class StructProcessData;
class StructEntityData;
class DescTimeSeries;
class EntityInstanceOS;
class DescriptorOS;
class ParameterOS;
class MDescOSParticipation;
class MultipleDescriptorOS;
class OutputSpec;
class ProcessOutputSpec;
class EventOutputSpec;

class StructProcessAction;
struct StructField;

  extern Tableau<Event*>* constCheckingTab;
  extern Tableau<Event*>* destCheckingTab;
  extern Tableau<Process*>* allocationCheckingTab;

/**@name Types prédéfinis, variable globales, constantes numériques et symboliques
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreTypes.h
[/P]
\end{itemize} 
*/
//@{  // ---------- fermeture dans UserTypes.h ----------


/**@name Définitions de types par {\em typedef}
 */

//@{

/**@name Les types d'identificateurs de classes

Toutes les classes d'objets (Entity, Process, \ldots) prédéfinies dans \BDb sont dotées d'un identificateur numérique (entier) caractéristique. Comme ces entiers sont associés, {\em via} des énumérations (enum), à des noms symboliques, les identificateurs sont couramment appellés 'symboles de classe'. Par exemple, l'identificateur de la classe des animaux pourra être 124, correspondant au symbole ANIMAL dans l'énumération des identificateurs des classes d'entités.\\

\begin{itemize}
\item Le type des identificateurs de classes d'entités : {\em UserClassId}.\index{UserClassId}
\item Le type des identificateurs de classes de descripteurs : {\em UserDescriptorId}.\index{UserDescriptorId}
\item Le type des identificateurs de classes de méthodes : {\em UserMethodId}.\index{UserMethodId}
\item Le type des identificateurs de classes d'arguments : {\em UserArgumentId}.\index{UserArgumentId}
\item Le type des identificateurs de classes de processus : {\em UserProcessId}.\index{UserProcessId}
\item Le type des identificateurs de classes de processus : {\em UserEventId}.\index{UserEventId}
\item Le type des identificateurs de classes de fichiers : {\em UserFileId}.\index{UserFileId}
\end{itemize}

Les types ci-dessus sont des alias de {\em int}. Ils sont définis pour une clarification sémantique : si une méthode possède un argument de type {\em UserClassId}, on doit comprendre que l'entier valeur de l'argument est un identificateur de classe d'entités. Cependant, les valeurs pourront être déclarées du type {\em int}, comme dans l'exemple ci-dessous, ou de n'importe quel alias de {\em int} (un type énuméré par exemple).\\

[P]
\begin{verbatim}
float f(UserClassId id) {
  ...
};
int x = 124;
float y = f(x);
\end{verbatim}
[/P]

Les symboles (de classe) prédéfinis dans \BD sont répertoriés dans la page 'Types prédéfinis ... / Constantes symboliques générales à tous les domaines'.
 */

//@{
typedef int UserClassId;
typedef int UserProcessId;
typedef int UserEventId;
typedef int UserDescriptorId;
typedef int AppDescriptorId;
typedef int UserMethodId;
typedef int UserArgumentId;
//@}  // fin identificateurs de classes

/**@name Les types de valeurs

\begin{itemize}
\item Le type pseudo-booléen : {\em Boolean} 

{\em Boolean} n'est qu'un alias de {\em int}. On ne doit l'attribuer qu'à des variables ou des fonctions dont les seules valeurs sont 0 et toute autre valeur sémantiquement équivalente à 1.\index{Boolean}\\
A la place des valeurs 0 et (par exemple) 1, on peut utiliser les symboles FALSE et TRUE, respectivement.\index{FALSE}\index{TRUE}\\
Ne pas confondre avec le type C++ {\em bool}.
\end{itemize}

\begin{itemize}
\item Le type "tableau d'entiers" : {\em TableauInt} ou {\em pIntTab}.\index{TableauInt}
\item Le type "tableau de flottants simple précision" : {\em TableauFloat} ou {\em pFloatTab}.\index{TableauFloat}  
\item Le type "tableau de flottants double précision" : {\em TableauDouble} ou {\em pDoubleTab}.\index{TableauDouble} 
\item Le type "tableau d'identificateurs de classes d'entités" : {\em UserClassIdTab}.\index{UserClassIdTab}
\item Le type "tableau d'entités" : {\em pEntityTab}.\index{pEntityTab}
\item Le type "tableau de pointeurs sur des tableaux d'entités" : {\em pEntityDTab}.\index{pEntityDTab}
\item Le type "tableau de spécifications d'ensemble d'entités" : {\em pEntitySpecTab}.\index{pEntitySpecTab}
\item Le type "tableau de spécifications de domaine de valeurs" : {\em pDescValueSpecTab}.\index{pDescValueSpecTab}
\item Le type "tableau de descripteurs constants d'entités" : {\em pConstantDescriptorTab}.\index{pConstantDescriptorTab}
\item Le type "tableau de descripteurs variables d'entités" : {\em pVariableDescriptorTab}.\index{pVariableDescriptorTab}
\item Le type "tableau de structures {processus, action}" : {\em StructProcessActionTab}.\index{StructProcessActionTab}
\item Le type "liste de couples {date, valeur} pour les descripteurs variables" : {\em pStructClockValueList}.\index{pStructClockValueList}
\item Le type "tableau d'événements" : {\em pEventTab}.\index{pEventTab} 
\item Le type "tableau d'intervalles d'entiers" : {\em pIntIntervalTab}.\index{pIntIntervalTab}
\item Le type "tableau d'intervalles de flottants" : {\em pFloatIntervalTab}.\index{pFloatIntervalTab}
\item Le type "tableau d'intervalles de flottants double précision" : {\em pDoubleIntervalTab}.\index{pDoubleIntervalTab} 
\end{itemize}

Sauf dans le cas des entiers, les tableaux ou listes contiennent des pointeurs sur des instances des classes correspondantes.\\

Ces types sont définis par des instructions dont voici deux exemples :

[P]
\begin{verbatim}
typedef int                   Boolean; 
typedef Tableau<BasicEntity*> pEntityTab;
\end{verbatim}
[/P]

*/
//@{

typedef int Boolean; 
//#ifndef __GNUG__
//   #define _bool_h 1                  // interdit l'inclusion de <bool.h>
//   enum bool { false=0,FALSE=0,true=1,TRUE=1};  // ajout Barbier 970515 
//#else 
  #ifndef TRUE
     #define FALSE 0 
     #define TRUE 1  
     #define false 0  // + JPR 030613
     #define true 1   // + JPR 030613 
  #endif   
//#endif

typedef Tableau<char*>                TableauString;

typedef Tableau<DescriptedEntity*>    pDescriptedEntityTab;
typedef Tableau<BasicEntity*>         pEntityTab;
typedef Tableau<pEntityTab*>          pEntityDTab;

typedef Tableau<int>                  pIntTab; 
typedef Tableau<int>                  TableauInt;
typedef Tableau<int>                  UserClassIdTab;
typedef Tableau<int>                  ArgumentIdTab;
typedef Tableau<float>                pFloatTab;
typedef Tableau<float>                TableauFloat;
typedef Tableau<double>               pDoubleTab;
typedef Tableau<double>               TableauDouble;
typedef Tableau<pIntTab*>             pIntDTab; 
typedef Tableau<pFloatTab*>           pFloatDTab; 

typedef Tableau<Event*>               pEventTab;
typedef Tableau<Process*>             pProcessTab; // cacher (used in RunCore)
typedef Tableau<EntitySpec*>          pEntitySpecTab;  
typedef Tableau<ConstantDescriptor*>  pConstantDescriptorTab;
typedef Tableau<VariableDescriptor*>  pVariableDescriptorTab;
typedef Tableau<MetaClass*>           pMetaClassTab; 
typedef Tableau<StructureMonitor*>    pStructureMonitorTab;   
typedef Tableau<DescValueMonitor*>    pDescValueMonitorTab; 
typedef Tableau<Parameter*>           pParameterTab;  
typedef Tableau<StructProcessAction*> StructProcessActionTab;
typedef Tableau<File*>                pFileTab;
typedef Tableau<OutputSpec*>          pOutputSpecTab; // + JPR 071109
typedef Tableau<StructField*>         pFieldTab;
typedef Tableau<Argument*>            pArgumentTab;
typedef Tableau<Method*>              pMethodTab; 
typedef Tableau<DescTimeSeries*>      pDescTimeSeriesTab;
typedef Tableau<DescValueSpec*>       pDescValueSpecTab;  
  

typedef Liste<ClockValueItem*>        pClockValueList;
typedef Liste<ChangeStructureItem*>   pChangeStructureList;
typedef Liste<EntityInstanceOS*>      pEntityInstanceOSList;
typedef Liste<DescriptorOS*>          pDescriptorOSList;
typedef Liste<ParameterOS*>           pParameterOSList;
typedef Liste<MultipleDescriptorOS*>  pMultipleDescriptorOSList;
typedef Liste<MDescOSParticipation*>  pMDescOSParticipationList;
                                      
typedef Liste<OutputSpec*>            pOutputSpecList;
typedef Liste<StructEventData*>       pStructEventDataList;
typedef Liste<StructProcessData*>     pStructProcessDataList;
typedef Liste<Entity*>                pEntityList;
typedef Liste<Event*>                 pEventList;
//@}  // fin types de valeurs

//@} // fin typedef 


/**@name Le type des valeurs multiformes : {\em UnifiedValue}

Une variable peut être déclarée de ce type si elle peut prendre des valeurs de types différents à des moments différents (mais bien entendu une valeur d'un type unique à un moment donné). Si {\em x} est une variable de ce type, on accède à la valeur supposée entière par [/P]x.intValue[/P]. Noter que C ne fait pas de contrôle de type.\index{UnifiedValue}\label{defUnifiedValue}\\

Les autres accesseurs aux valeurs typées sont : [P]x.floatValue[/P], [P]x.doubleValue[/P], [P]x.entityValue,[/P], [P]x.dateValue[/P] (pour une valeur de type {\em Date*}) et [P]x.stringValue[/P].

[P]
\begin{verbatim}
{
  ...
  UnifiedValue U;
  U.stringValue = "uneChaine";
  printf("\nla valeur de U est '%s'", U.stringValue);
  U.intValue = 1;
  printf("\nla valeur de U est %d", U.intValue);
  ...
}
\end{verbatim}
[/P]
{\small
\begin{verbatim}
>>
la valeur de U est 'uneChaine'
la valeur de U est 1
\end{verbatim}
}
*/
//@{ 

union UnifiedValue {
        //
  int           intValue;
        //
  float         floatValue;
        //
  double        doubleValue;
        //
  BasicEntity*  entityValue;
        //
  Date*         dateValue;   // + JPR 080822
        //
  char*         stringValue;  // déplacement JPR 080826 suite pb lect SDF
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const UnifiedValue&) const {return TRUE;};
  Boolean         operator > (const UnifiedValue&) const {return TRUE;};
  //
  // Les 3 opérateurs suivants sont déclarés pour le besoin de templateTableau.
  // Leur utilisation est proscrite.
  //
  Boolean operator==(const UnifiedValue& objr) const {
	return TRUE;
  };
  Boolean operator!=(const UnifiedValue& objr) const {
	return FALSE;
  };
  /*
ostream& operator << (ostream& os, const UnifiedValue& obj) const {
	os << obj.intValue << '\n';
	return os;
  };
  */
};
//@} // fin  valeurs multiformes

/**@name Variables globales (dont options de trace)
Toutes ces variables sont réferençables de n'importe où dans le code spécifique au domaine d'application.
 */
//@{
         ///Pointeur \index{pCurrentSim}sur l'instance de [C]Simulation[/C] en cours. La variable ici déclarée est réalisée dans le fichier [F]CoreTypes.cc[/F]. La création de l'instance est faite normalement en début du programme principal : [P]\begin{verbatim}Simulation* pCurrentSim = new Simulation();\end{verbatim}[/P]\label{pCurrentSim1}
extern Simulation* pCurrentSim;

         ///Numéro d'identification de la simulation. Variable globale éventuellement valuée par une option de la ligne de commande (-IDN). Permet d'identifier, par un numéro caractéristique de la simulation dont ils sont issus, les résultats écrits sur des fichiers externes à partir des spécifications de sorties.
extern int gCurrentSimNumId; // JPR 080204

         ///Semence pour la génération de nombres aléatoires. Variable globale de type 'entier non signé'. C'est l'argument à utiliser dans tous les appels à la fonction [Y]srand[/Y]. On donne une valeur à [Y]gRandomSeed[/Y] dans le fichier des paramètres de la simulation, ou dans le fichier des directives de la simulation, ou bien n'importe o] dans le code de l'application, notamment dans le programme principal.\index{gRandomSeed} 
extern unsigned int gRandomSeed;

         ///Conteneur anonyme multi-usages de chaînes de caractères. Variable globale de type 'chaîne de caractères'. Permet de construire des chaînes temporaires et accessibles de toute autre partie du programme. La taille maximale de la chaîne est {\em 100}. La chaîne peut notamment être construite par la fonction C [Y]sprintf(gGlobalString, <format>, <liste>)[/Y].\index{gGlobalString} 
extern char gGlobalString[100];

         // Variable globale permettant de diriger des sorties vers un fichier (notamment des messages divers en phase de débuggage) pendant que les printf et cout réalisent le dialogue avec l'IHM : pas de mélange des deux sortes de messages. Dans le [Y]main()[/Y], faire [P]pTmpFile = fopen("uneTrace", "w"); ... fclose(pTmpFile);[/P].
extern FILE* pTmpFile;  

         /// Variable globale permettant de diriger les avertissements (les captages d'exception qui ne provoquent pas l'arr\^et de l'éxécution) vers un fichier. Ce fichier peut \^etre lu après l'éxécution. L'ouverture est faite lors de la lecture de la ligne "OUT_DIR <path>;" dans le fichier des paramètres de la simulation. Dans le [Y]main()[/Y], faire [P]pLogFile = fopen(".log", "w"), ... fclose(pLogFile);[/P].
extern FILE* pLogFile;  

         /// Variable globale entière fixant la taille maximale des champs de type {\em char*} d'un enregistrement de fichier. Cette taille est fixée par défaut à 50 (y compris le caractère de fin de chaîne). On peut modifier la valeur de [Y]gFieldMaxSize[/Y] n'importe où dans le code de l'application, notamment dans le programme principal.\index{gFieldMaxSize}
extern int gFieldMaxSize; 

/// Traducteur des symboles de classe d'entités en chaînes de caractères. Chaque élément de ce tableau, d'indice {\small {\tt k}}, contient en format {\em char*}, les caractères de l'élément d'énumération des symboles dont l'entier correspondant est {\small {\tt k}.
extern Tableau<char*> ClassIdToCharTab;
extern int gClassIdToCharTabSize;

/// Traducteur des symboles de classe de descripteurs en chaînes de caractères. Lire le commentaire pour [Y]ClassIdToCharTab[/Y].
extern Tableau<char*> DescIdToCharTab;
extern int gDescIdToCharTabSize;

/// Traducteur des symboles de classe de méthodes en chaînes de caractères. Lire le commentaire pour [Y]ClassIdToCharTab[/Y]. 
extern Tableau<char*> MethodIdToCharTab;
extern int gMethodIdToCharTabSize;

/// Traducteur des symboles de classe d'arguments en chaînes de caractères. Lire le commentaire pour [Y]ClassIdToCharTab[/Y]. 
extern Tableau<char*> ArgumentIdToCharTab;
extern int gArgumentIdToCharTabSize;

/// Traducteur des symboles des jours de la semaine en chaînes de caractères (français). Voir l'énumération {\em ShortWeekDayName}.
extern Tableau<char*> ShortWeekDayNameToCharTab;   // + JPR 101208
void D_MakeShortWeekDayNameToCharTab();

/// Traducteur des symboles des jours de la semaine en chaînes de caractères (anglais). Voir l'énumération {\em ShortWeekDayName_e}.
extern Tableau<char*> ShortWeekDayNameToCharTab2;   // + JPR 110624
void D_MakeShortWeekDayNameToCharTab2();

/// Traducteur des symboles des mois en chaînes de caractères (français). Voir l'énumération {\em ShortMonthName}
extern Tableau<char*> ShortMonthNameToCharTab;   // + JPR 101208
void D_MakeShortMonthNameToCharTab();

/// Traducteur des symboles des mois en chaînes de caractères (anglais). Voir l'énumération {\em ShortMonthName_e}
extern Tableau<char*> ShortMonthNameToCharTab2;   // + JPR 110624
void D_MakeShortMonthNameToCharTab2();

/**@name Variables globales pour la taille des énumérations
Indiquent le plus grand numéro symbolique dans les énumérations {\em UserClassId}, {\em UserDescriptorId}\ldots \\

Les valeurs numériques des constantes symboliques dans les énumérations complémentaires dans une sous-couche immédiate de DIESE doivent être supérieures à ces valeurs. Ces valeurs doivent y être surchargées avec les plus grandes valeurs attribuées dans la sous-couche. Ces règles doivent être à nouveau appliquées pour une sous-couche encore inférieure. Ci-dessous, les valeurs attribuées dans la couche \BD.\\
       
[Y]g_BD_ReservedUserClassId[/Y]      :  6\\
[Y]g_BD_ReservedUserDescriptorId[/Y] : 13\\
[Y]g_BD_ReservedUserMethodId[/Y]     : 61\\
[Y]g_BD_ReservedUserArgumentId[/Y]   : 44\\
[Y]g_BD_ReservedUserProcessId[/Y]    : -9\\
[Y]g_BD_ReservedUserEventId[/Y]      :  3\\
[Y]g_BD_ReservedUserFileId[/Y]       : -9\\
*/ 
//@{
         //
extern int g_BD_ReservedUserClassId;
         //
extern int g_BD_ReservedUserDescriptorId;
         //
extern int g_BD_ReservedUserMethodId;
         //
extern int g_BD_ReservedUserArgumentId;
         //
extern int g_BD_ReservedUserProcessId;
         //
extern int g_BD_ReservedUserEventId;
         //
extern int g_BD_ReservedUserFileId;

//@}  // fin var globales taille enum

/**@name Activation/désactivation de traces

\begin{itemize}
\item Trace de la gestion de l'agenda : \\
[P]
g_BD_TraceActionMode = TRACE_ACTION_OFF; // trace désactivée
g_BD_TraceActionMode = TRACE_ACTION_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-V'.\\ \\

\item Trace des constr./destr. des classes de DIESE : \\
[P]
g_BD_TraceNewDelMode = TRACE_NEWDEL_OFF; // trace désactivée
g_BD_TraceNewDelMode = TRACE_NEWDEL_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-M'.\\ \\

\item Synthèse des nombres de constr./destr. des classes de DIESE : \\
[P]
g_BD_MemoryReportMode = MEM_REPORT_OFF; // trace désactivée
g_BD_MemoryReportMode = MEM_REPORT_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-MR'.\\ \\

\item Affichage de messages d'avertissements divers : \\
[P]
g_BD_TraceWarnings = TRACE_WARN_OFF; // trace désactivée
g_BD_TraceWarnings = TRACE_WARN_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-Warn'.\\ \\

\item Trace des constr./destr. des tableaux : \\
[P]
g_BD_TraceTableauMode = TRACE_TABLEAU_OFF;  // trace désactivée
g_BD_TraceTableauMode = TRACE_TABLEAU_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-T'.\\ \\

\item Trace des constr./destr. des listes : \\
[P]
g_BD_TraceListeMode = TRACE_LISTE_OFF;  // trace désactivée
g_BD_TraceListeMode = TRACE_LISTE_ON;  // trace activée
[/P]

La trace peut être activée par l'option de commande '-L'. \\ \\

\item Trace de la lecture des fichiers en arguments de la commande\\
Exemple de la trace de la lecture du fichier des paramètres du système :\\
[P]
g_BD_TraceParsingP = TRACE_PARSING_OFF;  // trace désactivée
g_BD_TraceParsingP = TRACE_PARSING_ON;  // trace activée
[/P]

La trace peut être activée par la directive {\em TRACE PARSING;} dans le fichier dont on veut tracer l'interprétation. \\ \\
Les autres variables sont : [Y]g_BD_TraceParsingI[/Y], [Y]g_BD_TraceParsingO[/Y], [Y]g_BD_TraceParsingS[/Y], [Y]g_BD_TraceParsingD[/Y], pour, respectivement, le fichier de paramètres de la simulation, des spécifications de sortie, de la structure du système, des directives de simulation. \\ \\

De manière générale, l'option par défaut est la désactivation de la trace.

\end{itemize}
*/
//@{
#ifndef _GTRACEACTIONMODE_H
#define _GTRACEACTIONMODE_H
enum boolTraceActionMode {TRACE_ACTION_OFF, TRACE_ACTION_ON};
extern boolTraceActionMode gTraceActionMode;
extern boolTraceActionMode g_BD_TraceActionMode;
#endif  // _GTRACEACTIONMODE_H

#ifndef _GTRACENEWDELMODE_H
#define _GTRACENEWDELMODE_H
enum boolTraceNewDelMode {TRACE_NEWDEL_OFF, TRACE_NEWDEL_ON};
extern boolTraceNewDelMode gTraceNewDelMode;
extern boolTraceNewDelMode g_BD_TraceNewDelMode;
#endif  // _GTRACENEWDELMODE_H

#ifndef _GMEMREPORTMODE_H
#define _GMEMREPORTMODE_H
enum boolMemoryReportMode {MEM_REPORT_OFF, MEM_REPORT_ON};
extern boolMemoryReportMode g_BD_MemoryReportMode;
#endif  // _GMEMREPORTMODE_H

//--------------------------------------------------  // + JPR 100803
#ifndef _GTRACEWARNINGS_H
#define _GTRACEWARNINGS_H
enum boolTraceWarnings {TRACE_WARN_OFF, TRACE_WARN_ON};
extern boolTraceWarnings g_BD_TraceWarnings;
#endif  // _GTRACEWARNINGS_H
//--------------------------------------------------

//--------------------------------------------------
//  - JPR 021022  -> Tableau.h Liste.h
#ifndef _GTRACETABLEAUMODE_H
#define _GTRACETABLEAUMODE_H
enum boolTraceTableauMode {TRACE_TABLEAU_OFF, TRACE_TABLEAU_ON};
extern boolTraceTableauMode gTraceTableauMode;
extern boolTraceTableauMode g_BD_TraceTableauMode;
extern int gNbConstTableau;   // 981126
extern int gNbDestTableau;    // 981126
#endif  // _GTRACETABLEAUMODE_H

#ifndef _GTRACELISTEMODE_H
#define _GTRACELISTEMODE_H
enum boolTraceListeMode {TRACE_LISTE_OFF, TRACE_LISTE_ON};
extern boolTraceListeMode gTraceListeMode;
extern boolTraceListeMode g_BD_TraceListeMode;
extern int gNbConstCell;   
extern int gNbDestCell;  
extern int gNbConstListe;   
extern int gNbDestListe;
#endif  // _GTRACELISTEMODE_H

//--------------------------------------------------
extern int g_BD_gmtoff;   // + JPR 101129
//--------------------------------------------------

#ifndef _GTRACEPARSING_H
#define _GTRACEPARSING_H
enum boolTraceParsing {TRACE_PARSING_OFF, TRACE_PARSING_ON}; 
                         // trace du parsing en préambule de la simulation
extern boolTraceParsing g_BD_TraceParsingP;
extern boolTraceParsing g_BD_TraceParsingI;
extern boolTraceParsing g_BD_TraceParsingO;
extern boolTraceParsing g_BD_TraceParsingS;
extern boolTraceParsing g_BD_TraceParsingD;
#endif  // _GTRACEPARSING_H

//--------------------------------------------------

//@}  // fin var trace

//@}  // fin var globales

/*
La variable ci-dessous sert à l'externalisation des corps de méthodes.
Un appel Body(Method* pM) ne peut en effet être passé à un interpréteur externe par "swiggisation" que si l'argument est une variable globale.
De façon générale, on ne peut swigger une fonction que si elle n'a que des arguments d'un type de base de C, ou bien d'un autre type à condition que l'argument soit une variable globale.
Un alternative à la "globalisation" de pM aurait été de transformer l'appel Body(Method* pM) en un appel Body(char* s, char* t) avec 
s : nom de classe du Process ou de l'Event, ou nom d'instance de l'Entity
t : nom de classe de la Method
*/ 

#ifdef _SWIG_
extern 
  EntityMethod* pSwigEntityMethod;
extern 
  ProcessMethod* pSwigProcessMethod;
extern 
  EventMethod* pSwigEventMethod;
extern 
  MonitorMethod* pSwigMonitorMethod;
extern 
  EntitySpecMethod* pSwigEntitySpecMethod;

extern int perlIntReturnValue;
extern bool perlBoolReturnValue;
extern float perlFloatReturnValue;
extern double perlDoubleReturnValue;
extern char* perlStringReturnValue;
extern BasicEntity* perlpEntityReturnValue;
extern pEntityTab* perlpEntityTabReturnValue;
extern pEntityTab perlEntityTabReturnValue;
extern pIntTab perlTabIntReturnValue;
extern DescValueSpec* perlpDescValueSpecReturnValue;
extern pDescValueSpecTab* perlpDescValueSpecTabReturnValue;
#endif  //_SWIG_


/**@name Constantes numériques générales
Ces variables ont, respectivement, la valeur du plus grand entier, du plus petit entier, etc., sur la machine de l'utilisateur.
 */
//@{
         // -9
extern int D_INT_UNSIGNIFICANT;
         // -999.0 
extern float D_FLOAT_UNSIGNIFICANT; 
         /// Le plus grand entier (positif). 
extern int D_INTMAX; 
         /// Le plus petit entier (négatif).
extern int D_INTMIN;  
         /// Le plus grand flottant (positif)
extern float D_FLOATMAX;
         /// Le plus petit flottant (négatif).
extern float D_FLOATMIN;    
         /// Le plus petit flottant positif.
extern float D_float_EPS;   
         /// Le plus grand flottant en double précision (positif).
extern double D_DOUBLEMAX;
         /// Le plus petit flottant en double précision (négatif).
extern double D_DOUBLEMIN;  
         /// Le plus petit flottant positif en double précision.
extern double D_double_EPS;

//@}  // fin Constantes numériques générales

/**@name Constantes symboliques ({\em enum}) générales à tous les domaines
Les constantes symboliques, qui font l'objet des {\em énumérations}\index{énumérations} ci-dessus) sont en fait des nombres entiers positifs associés à des symboles (un par nombre). Il est alors équivalent de faire référence à la valeur numérique où à sa traduction symbolique. Par exemple, les deux messages suivants sont équivalents : [P]IsInstanceOf(2)[/P] et [P]IsInstanceOf(BD\_ENTITY)[/P] (voir l'énumération 'Types d'objets').
 */ 
//@{

          /**@name  Types d'objets.
\index{BD\_BASIC\_ENTITY} \index{BD\_DESCRIPTED\_ENTITY} \index{BD\_ENTITY} \index{BD\_PROCESS} \index{BD\_EVENT} \index{BD\_MONITOR} \index{BD\_ENTITYSPEC}

BD\_BASIC\_ENTITY (= 0) est le symbole de classe de la classe mère des entités, non munies de descripteurs ni de méthodes. Ce symbole (ou l'entier correspondant) peut être utilisé comme argument de certaines fonctions, telles celles renvoyant des informations sur les classes elles-mêmes : [Y]GetNumberOfEntities[/Y](BD\_ENTITY).\label{enumEntityClassId}\\

BD\_DESCRIPTED\_ENTITY (= 1) est le symbole de classe de la classe des entités possiblement munies de descripteurs.\\

BD\_ENTITY (= 2) est le symbole de classe de la classe des entités possiblement munies de descripteurs et de méthodes.\\

BD\_ENTITYSPEC (= 3) est le type des méthodes dont l'objet est une spécification d'ensemble d'entités.\\

BD\_EVENT (= 4) est le type des méthodes dont l'objet est un événement.\\

BD\_MONITOR (= 5) est le type des méthodes dont l'objet est une instance de [C]DescValueMonitor[/C] ou de [C]StructureMonitor[/C].\\

BD\_PROCESS (= 6) est le type des méthodes (instances de [C]Method[/C]) dont l'objet est un processus.
*/
//@{
enum ObjectClassId {
  //
  // si modif, alors modifier aussi MakeClassIdToCharTab()
  //
BD_BASIC_ENTITY=0,
BD_DESCRIPTED_ENTITY,
BD_ENTITY,
BD_ENTITYSPEC, 
BD_EVENT,   
BD_MONITOR,
BD_PROCESS=6
};
//@}


         /**@name Types de valeurs.
VOID=0,\\ 
BOOLEAN, INT, FLOAT, DOUBLE, STRING,\\ 
P\_ENTITY, P\_ENTITYSPEC, P\_DESCVALUESPEC,\\ 
INTDOMAIN, FLOATDOMAIN, DOUBLEDOMAIN,\\ 
ENTITY\_TAB, DESCVALUESPEC\_TAB,\\ 
INT\_TAB, FLOAT\_TAB, DOUBLE\_TAB, STRING\_TAB,\\
P\_EVENT,\\ 
P\_DESCRIPTOR\index{ValueTypeId}\\

Chaque élément de cette énumération caractérise une ou plusieurs des classes d'objets suivantes, en indiquant quel est le type C++ de la valeur associée aux instances de la classe (un exemple est donné pour chacune) :
\begin{itemize} 

  \item descripteurs (classe [C]Descriptor[/C]) : 
  \begin{itemize} 
  \item si déclaré INT, le type C++ de la valeur du descripteur est {\em int}  
  \item si déclaré P\_ENTITY, le type C++ de la valeur du descripteur est {\em BasicEntity*}
  \end{itemize}

\item méthodes (classe [C]Method[/C]) :  
  \begin{itemize} 
  \item si déclarée BOOLEAN, le type C++ de la valeur renvoyée par le corps de la méthode est {\em bool}  
  \item si déclarée P\_EVENT, le type C++ de la valeur est {\em Event*}
  \end{itemize}

\item arguments de méthodes (classe [C]Argument[/C]) :   
  \begin{itemize} 
  \item si déclaré FLOAT, le type C++ de la valeur de l'argument est {\em float}  
  \item si déclaré P\_DESCRIPTOR, le type C++ de la valeur est {\em Descriptor*}
  \end{itemize}

\item fichier de données (classe [C]File[/C]) :    
  \begin{itemize} 
  \item si un champ des enregistrements est déclaré STRING, le type C++ de la valeur du champ est {\em char*}
  \end{itemize}

\item paramètre (classe [C]Parameter[/C]) :    
  \begin{itemize} 
  \item si déclaré DOUBLE, le type C++ de la valeur du paramètre est {\em double}
  \end{itemize}

\end{itemize}
P\_ENTITYSPEC, P\_DESCVALUESPEC, sont d'autres types qui caractérisent des objets dont la valeur est un {\em pointeur} sur, respectivement, une spécification d'ensemble d'entités, une spécification de domaines de valeurs.\\

INTDOMAIN, FLOATDOMAIN, DOUBLEDOMAIN, STRING\_TAB, INT\_TAB, FLOAT\_TAB, ENTITY\_TAB et DESCVALUESPEC\_TAB sont d'autres types qui caractérisent des objets dont la valeur est un {\em pointeur} sur, respectivement, un tableau d'intervalles d'entiers, un tableau d'intervalles de valeurs flottantes simple et double précision, un tableau de chaînes de caractères, un tableau d'entiers, un tableau de flottants simple précision, un tableau de pointeurs sur des entités et un tableau de pointeurs sur des spécifications de domaines de valeurs.
*/
//@{
enum ValueTypeId {
   VOID=0    ,
   BOOLEAN   ,
   INT       ,
   FLOAT     ,
   DOUBLE    ,
   STRING    ,
   P_ENTITY  ,
   ENTITY_TAB,
   P_ENTITYSPEC,
   INTDOMAIN,
   FLOATDOMAIN,
   DOUBLEDOMAIN,
   P_DESCVALUESPEC,
   DESCVALUESPEC_TAB,
   STRING_TAB,
   INT_TAB,           // JPR 090421
   FLOAT_TAB,         // JPR 051114
   DOUBLE_TAB,        // JPR 061018
   P_EVENT,
   P_DESCRIPTOR,
   UNSPECIFIED_VALUETYPE
};
//@}

         /**@name Catégories de descripteurs.
CONSTANT=1, VARIABLE \\

Enumération exhaustive des catégories de descripteurs des entités. \\

Les descripteurs sont constants ou variables. Un {\em descripteur constant} se voit affecter en début de simulation une valeur qui n'est pas destinée à être modifiée par la suite, au contraire d'un {\em descripteur variable} dont la valeur, typiquement, est modifiée par un processus.\\

La classe de base [C]Descriptor[/C] possède autant de sous-classes qu'il y a de sous-classes dans l'énumération ci-dessus (2) croisées avec les types des valeurs. Dans la présente version, la valeur d'un descripteur est soit d'un des quatre types {\em int}, {\em float}, {\em double} et {\em char*} de C, soit une instance d'une sous-classe d'[C]Entity[/C], soit encore un tableau d'instances d'[C]Entity[/C]\ldots (voir la page 'Constantes symboliques ... / Types de valeurs').\\

Une classe de descripteurs définie par l'utilisateur pour une application particulière {\em descend} de l'une des sous-classes de [C]Descriptor[/C].\\ 
Le message [Y]SubClass()[/Y] adressé, par exemple, à une instance d'une sous-classe quelconque de [C]IntConstantDescriptor[/C] renvoie la valeur CONSTANT.\\ 
Le message [Y]ClassType()[/Y] adressé à la même instance renvoie la valeur INT.\index{DescriptorSubclassId}
*/
//@{
enum DescriptorSubclassId {
CONSTANT=1, VARIABLE
};
//@}


         /**@name Classes des descripteurs prédéfinis d'entités.
Enumération exhaustive des descripteurs définis dans \D.\\

Certains sont exploités dans \Db (dans la classe [C]DescriptedEntity[/C]). Ils sont alors hérités par toutes les sous-classes de DescriptedEntity. Les autres peuvent être exploités dans les sous-classes de DescriptedEntity propres aux applications.\\

Avec chaque symbole, on indique ci-dessous la classe de descripteur qui le possède, le type de la valeur du descripteur et sa sémantique proposée.\index{D_DescriptorId}\index{Descripteurs prédéfinis}\\

{\large Constants :}\\
\begin{itemize}
\item TYPE\_ATTRIBUTE=1\\
\DTab4 Classe [C]TypeAttribute[/C]. Type {\em int}. Le type d'une entité. Différencie les instances d'une classe sans avoir à identifier des sous-classes. Identifie de manière commode les instances d'une sous-classe.\\
\item INSTANT\_ATTRIBUTE=2\\
\DTab4 Classe [C]InstantAttribute[/C]. Type {\em int} (valeur d' l'horloge de la simulation). La valeur est un point sur l'axe du temps simulé.\\
\item DESCRIPTOR\_ID\_ATTRIBUTE=3\\
\DTab4 Classe [C]DescriptorIdAttribute[/C]. Type {\em int}. La valeur est le symbole de classe d'un descripteur.\\
\item DESCRIPTOR\_NAME\_ATTRIBUTE=4\\
\DTab4 Classe [C]DescriptorNameAttribute[/C]. Type {\em char*}. La valeur est le nom de classe d'un descripteur.\\
\item ENTITY\_ID\_ATTRIBUTE=5\\
\DTab4 Classe [C]EntityIdAttribute[/C]. Type {\em int}. La valeur est le symbole de classe d'une entité.\\
\item ENTITY\_NAME\_ATTRIBUTE=6\\
\DTab4 Classe [C]EntityNameAttribute[/C]. Type {\em char*}. La valeur est le nom de classe d'une entité.\\
\item TIME\_DOMAIN\_ATTRIBUTE=7\\
\DTab4 Classe [C]TimeDomainAttribute[/C]. Type {\em pIntIntervalTab}. La valeur est un intervalle sur les entiers, typiquement un intervalle de temps.\\
\end{itemize}

Les descripteurs suivants sont anonymes, c'est-à-dire que leurs valeurs ne sont pas relatives à la dimension indiquée par le nom du descripteur. Il peuvent notamment être utilisés pour décrire une entité par un vecteur générique, qui peut contenir des hauteurs, des poids, \ldots sans que cela soit déterminé définitivement et de manière unique au moment de la conception de l'entité.\\
\begin{itemize}
\item ANONYMOUS\_INT\_TAB\_ATTRIBUTE=8\\
\DTab4 Classe [C]AnonymousIntTabAttribute[/C]. Type {\em pIntTab*}. \\
\item ANONYMOUS\_FLOAT\_TAB\_ATTRIBUTE=9\\
\DTab4 Classe [C]AnonymousFloatTabAttribute[/C]. Type {\em pFloatTab*}. \\
\end{itemize}


{\large Variables :}\\
\begin{itemize}
\item CURRENT\_ELEMENT\_INDEX=12\\
\DTab4 Classe [C]CurrentElementIndex[/C]. {\em int}. Descripteur exploitable dans une procédure qui parcourt la liste des éléments d'une entité et qui a besoin de faire connaître à son environnement l'élément qu'elle est en train d'examiner ou de traiter.\\
\end{itemize}

Les descripteurs suivants sont anonymes, c'est-à-dire que leurs valeurs ne sont pas relatives à la dimension indiquée par le nom du descripteur. Il peuvent notamment être utilisés pour décrire une entité par une grandeur générique : par exemple la taille, laquelle taille peut être une hauteur, un poids, \ldots sans que cela soit déterminé définitivement et de manière unique au moment de la conception de l'entité.\\
\begin{itemize}
\item ANONYMOUS\_INT\_ATTRIBUTE=13\\
\DTab4 Classe [C]AnonymousIntAttribute[/C]. Type {\em int}. \\
\item ANONYMOUS\_FLOAT\_ATTRIBUTE=14\\
\DTab4 Classe [C]AnonymousFloatAttribute[/C]. Type {\em float}. \\
\item ANONYMOUS\_DOUBLE\_ATTRIBUTE=15\\
\DTab4 Classe [C]Anonymous>DoubleAttribute[/C]. Type {\em double}. \\
\item ANONYMOUS\_STRING\_ATTRIBUTE=16\\
\DTab4 Classe [C]AnonymousStringAttribute[/C]. Type {\em char*}. \\
\item ANONYMOUS\_ENTITY\_ATTRIBUTE=17\\
\DTab4 Classe [C]AnonymousEntityAttribute[/C]. Type {\em BasicEntity*}. \\
\end{itemize}


*/
/* descripteurs plus utilisés
\item INT\_DESCRIPTOR\_FOR\_SORTING=10\\

\DTab4 Classe [C]IntDescriptorForSorting[/C]. Entier. Descripteur attaché à toutes les instances de [C]DescriptedEntity[/C], utilisé notamment par les fonctions [Y]SortUp[/Y], [Y]SortDown[/Y], [Y]MinValued[/Y] et [Y]MaxValued[/Y] (voir page 'Tri d'un tableau d'entités'). Elles lui donnent comme valeur la valeur entière du descripteur numérique qui sert de crtère de tri.\\
\item FLOAT\_DESCRIPTOR\_FOR\_SORTING=11\\

\DTab4 Classe [C]FloatDescriptorForSorting[/C]. Flottant. Comme ci-dessus. Les fonctions évoquées lui donnent comme valeur la valeur flottante du descripteur numérique qui sert de crtère de tri.\\
*/
//@{
enum D_DescriptorId {
  //
  // si modif, alors modifier aussi MakeDescIdToCharTab()
  //
//DESC_ID_FOR_SORTING=0,   // - JPR 110413
TYPE_ATTRIBUTE=1,
INSTANT_ATTRIBUTE=2,
DESCRIPTOR_ID_ATTRIBUTE=3,
DESCRIPTOR_NAME_ATTRIBUTE=4,
ENTITY_ID_ATTRIBUTE=5,
ENTITY_NAME_ATTRIBUTE=6,
TIME_DOMAIN_ATTRIBUTE=7,
ANONYMOUS_INT_TAB_ATTRIBUTE=8,
ANONYMOUS_FLOAT_TAB_ATTRIBUTE=9,
INT_DESCRIPTOR_FOR_SORTING=10,          // + JPR 110202
FLOAT_DESCRIPTOR_FOR_SORTING=11,        // + JPR 110202
CURRENT_ELEMENT_INDEX=12,
ANONYMOUS_INT_ATTRIBUTE=13,
ANONYMOUS_FLOAT_ATTRIBUTE=14,
ANONYMOUS_DOUBLE_ATTRIBUTE=15,
ANONYMOUS_STRING_ATTRIBUTE=16,
ANONYMOUS_ENTITY_ATTRIBUTE=17
};
//@}

         /**@name Classes de méthodes prédéfinies.
\begin{itemize}
\item Attachées à la classe [C]Process[/C]  \\
PROCESS\_PRECONDITION=1, PROCESS\_POSTPONEMENT\_DELAY,  \\
EXEC\_PROCESS, INITIALIZE\_PROCESS, GOONESTEPFORWARD\_PROCESS, STOP\_PROCESS, POST\_READ, \\

\item Attachées à la classe [C]Event[/C]  \\
EVENT\_POSTCONSEQUENCE=11, \\
GENERATE\_NEXTEVENT, UPDATE\_NEXTEVENT, DEFINE\_NEXTEVENTCLOCKTIME, \\
RANDOMIZE\_THISEVENTCLOCKTIME, RANDOMIZE\_NEXTEVENTCLOCKTIME, \\
IS\_POPPING\_TO\_SUSPEND, \\

\item Attachées à la classe [C]DescValueMonitor[/C]  \\
WHEN\_SET\_INT=21,WHEN\_SET\_FLOAT, WHEN\_SET\_DOUBLE, WHEN\_SET\_ENTITY, \\
WHEN\_SET\_ENTITYTAB, WHEN\_SET\_STRING, WHEN\_SET\_DESCVALUESPECTAB,
WHEN\_GET\_INT=31,WHEN\_GET\_FLOAT, WHEN\_GET\_DOUBLE, WHEN\_GET\_ENTITY, \\
WHEN\_GET\_ENTITYTAB, WHEN\_GET\_STRING, WHEN\_GET\_DESCVALUESPECTAB, \\
WHEN\_ADD\_ENTITY=41, WHEN\_ADD\_DESCVALUESPEC,  \\
WHEN\_REMOVE\_ENTITY, WHEN\_REMOVE\_DESCVALUESPEC, \\

\item Attachées à la classe [C]StructureMonitor[/C]  \\
WHEN\_ADD\_ELEMENT=48, WHEN\_REMOVE\_ELEMENT, \\

\item Attachées à la classe [C]EntitySpec[/C]  \\
SELECTOR\_PREDICATE=51, ENTITY\_INSTANTIATOR, ENTITYLIST\_INSTANTIATOR,  \\
QUANTITY\_EVALUATOR
\end{itemize}

Enumération exhaustive des symboles des classes de méthodes prédéfinies dans \D, dont l'existence est liée aux besoins du moteur de simulation. \\

Le rôle de ces méthodes, et corrolairement le contenu que doit donner l'utilisateur à leurs corps dans la couche applicative, est précisé dans la page dédiée à chaque classe.\index{CoreMethodId}\label{defCoreMethodId}
*/
//@{
enum CoreMethodId {
  //
  // si modif, alors modifier aussi MakeMethodIdToCharTab()
  //
  PROCESS_PRECONDITION=1, PROCESS_POSTPONEMENT_DELAY, EXEC_PROCESS, INITIALIZE_PROCESS, GOONESTEPFORWARD_PROCESS, STOP_PROCESS, POST_READ,
EVENT_POSTCONSEQUENCE=11, GENERATE_NEXTEVENT, UPDATE_NEXTEVENT, DEFINE_NEXTEVENTCLOCKTIME, RANDOMIZE_THISEVENTCLOCKTIME, RANDOMIZE_NEXTEVENTCLOCKTIME,
IS_POPPING_TO_SUSPEND,
WHEN_SET_INT=21,WHEN_SET_FLOAT, WHEN_SET_DOUBLE, WHEN_SET_ENTITY, WHEN_SET_ENTITYTAB, WHEN_SET_STRING, WHEN_SET_DESCVALUESPECTAB,
WHEN_GET_INT=31,WHEN_GET_FLOAT, WHEN_GET_DOUBLE, WHEN_GET_ENTITY, WHEN_GET_ENTITYTAB, WHEN_GET_STRING, WHEN_GET_DESCVALUESPECTAB,
WHEN_ADD_ENTITY=41, WHEN_ADD_DESCVALUESPEC, WHEN_REMOVE_ENTITY, WHEN_REMOVE_DESCVALUESPEC, 
WHEN_ADD_ELEMENT=48, WHEN_REMOVE_ELEMENT,
  SELECTOR_PREDICATE=51, ENTITY_INSTANTIATOR, ENTITYLIST_INSTANTIATOR, QUANTITY_EVALUATOR,
SPEC_SATISFIED=57
};
//@}

         /**@name Classes d'arguments prédéfinis.
\begin{itemize}

\item Attachés à aucune méthode dans \BD.  \\
DESCRIPTORID_ARGUMENT=1 : argument dont la valeur est un symbole de classe de descripteur  \\
CLOCK_VALUE_ARGUMENT=2 : argument dont la valeur est un point sur l'axe du temps simulé  \\

\item Attachés aux méthodes de la classe [C]Event[/C]  \\
INT_LOW_BOUND_ARGUMENT=11, INT_UPPER_BOUND_ARGUMENT,\\ 
PREVIOUS_EVENT_ARGUMENT,\\

\item Attachés aux méthodes de la classe [C]DescValueMonitor[/C]  \\
MONITOREDDESCRIPTOR_ARGUMENT=15,\\

CURRENTINTVALUE_ARGUMENT=21, CURRENTFLOATVALUE_ARGUMENT, \\
CURRENTDOUBLEVALUE_ARGUMENT, CURRENTSTRINGVALUE_ARGUMENT,\\
CURRENTENTITYVALUE_ARGUMENT, CURRENTENTITYTABVALUE_ARGUMENT,\\
CURRENTDESCVALUESPECTABVALUE_ARGUMENT,\\

CANDIDATEINTVALUE_ARGUMENT=31, CANDIDATEFLOATVALUE_ARGUMENT, \\
CANDIDATEDOUBLEVALUE_ARGUMENT, CANDIDATESTRINGVALUE_ARGUMENT,\\
CANDIDATEENTITYVALUE_ARGUMENT, CANDIDATEENTITYTABVALUE_ARGUMENT,\\
CANDIDATEDESCVALUESPECVALUE_ARGUMENT,\\
CANDIDATEDESCVALUESPECTABVALUE_ARGUMENT,\\

CURRENTENTITYCONTAINER_ARGUMENT=41, CANDIDATEENTITYELEMENT_ARGUMENT,\\

\item Attachés aux méthodes de la classe [C]EntitySpec[/C]  \\
CANDIDATEENTITY_ARGUMENT=44, REFERENCEENTITY_ARGUMENT, \\
CLASSID_ARGUMENT, LISTSIZE_ARGUMENT=44
\end{itemize}

Enumération exhaustive des symboles des classes d'arguments de méthodes, prédéfinis dans \Db et dont l'existence est liée aux besoins du moteur de simulation. \\ \index{CoreArgumentId}\label{defCoreArgumentId}

La sémantique de ces arguments, admise dans la couche \BD, est énoncée dans les pages dédiées à chaque classe.\\

Dèja attaché ou non à une classe de méthode dans la couche \BD, un argument prédéfini peut être attaché à une méthode de la couche applicative. Il est bon, dans ce cas, de respecter sa sémantique admise.
*/
//@{
enum CoreArgumentId {
DESCRIPTORID_ARGUMENT=1, CLOCK_VALUE_ARGUMENT,

INT_LOW_BOUND_ARGUMENT=11, INT_UPPER_BOUND_ARGUMENT, 
PREVIOUS_EVENT_ARGUMENT,
MONITOREDDESCRIPTOR_ARGUMENT,

CURRENTINTVALUE_ARGUMENT=21, CURRENTFLOATVALUE_ARGUMENT, 
CURRENTDOUBLEVALUE_ARGUMENT, CURRENTSTRINGVALUE_ARGUMENT,
CURRENTENTITYVALUE_ARGUMENT, CURRENTENTITYTABVALUE_ARGUMENT,
CURRENTDESCVALUESPECTABVALUE_ARGUMENT,

CANDIDATEINTVALUE_ARGUMENT=31, CANDIDATEFLOATVALUE_ARGUMENT, 
CANDIDATEDOUBLEVALUE_ARGUMENT, CANDIDATESTRINGVALUE_ARGUMENT,
CANDIDATEENTITYVALUE_ARGUMENT, CANDIDATEENTITYTABVALUE_ARGUMENT,
CANDIDATEDESCVALUESPECVALUE_ARGUMENT,CANDIDATEDESCVALUESPECTABVALUE_ARGUMENT,

CURRENTENTITYCONTAINER_ARGUMENT, CANDIDATEENTITYELEMENT_ARGUMENT,

CANDIDATEENTITY_ARGUMENT=41, REFERENCEENTITY_ARGUMENT, 
CLASSID_ARGUMENT, LISTSIZE_ARGUMENT
};
//@}


         /**@name Classes d'événements prédéfinis.
\begin{itemize}
\item CP_INIT_EVENT : événement [C]SetInitialState[/C] d'initialisation d'un processus continu
\item CP_PROCEED_EVENT : événement [C]ProceedFromCurrentState[/C] de continuation (sur un seul pas) d'un processus continu
\item CP_INIT_PROCEED_EVENT : événement [C]SetInitialStateAndProceed[/C] qui enchaîne l'initialisation et la première continuation (sur un seul pas) d'un processus continu
\item DP_EXEC_EVENT : événement [C]DiscreteExecution[/C] d'exécution d'un processus ponctuel
\end{itemize}

Enumération exhaustive des symboles de classe des événements gérés par le moteur de simulation. \index{EventClassId}


*/
//@{
enum EventClassId {
  CP_INIT_EVENT, CP_PROCEED_EVENT, CP_INIT_PROCEED_EVENT=2, DP_EXEC_EVENT=3
};
//@}


         /**@name Catégories de processus.
\begin{itemize}
\item DISCRETE_PROCESS : classe [C]DiscreteProcess[/C] des processus ponctuels\index{DISCRETE_PROCESS}
\item CONTINUOUS_PROCESS : classe [C]ContinuousProcess[/C] des processus continus\index{CONTINUOUS_PROCESS}
\end{itemize}

Enumération exhaustive des symboles de classe des sous-classes de processus gérés par le moteur de simulation. \\

Un processus est continu ou ponctuel. \\

Un processus qualifié ici de {\em ponctuel} \index{processus ponctuel (discret)}opère un changement d'état sur une période de durée nulle. Autrement dit, les états avant et après l'exécution du processus sont relatifs à la même date. Un tel processus peut être aussi qualifié de {\em discret}. \\

 Un processus qualifié ici de {\em continu}\index{processus continu} est caractérisé par une procédure qui opère un changement d'état entre deux dates différentes. La durée de la période séparant ces deux dates est appelée le {\em pas}\index{pas (d'un processus continu)} du procesus. Normalement, un processus continu revêt la forme d'un enchaînement de changements d'état opérés par la procédure qui le caractérise.\\

Les classes de processus définies par l'utilisateur pour une application particulière sont de l'un des types ci-dessus énumérés. A chaque type correspond une sous-classe prédéfinie de [C]Process[/C]. Le message [Y]ClassType()[/Y] adressé, par exemple, à une instance d'une sous-classe quelconque de [C]DiscreteProcess[/C] renvoie la valeur DISCRETE_PROCESS.\index{ProcessSubclassId}
*/
//@{
enum ProcessSubclassId {
DISCRETE_PROCESS=0, CONTINUOUS_PROCESS
};
//@}


         /**@name Types d'actions sur les processus.
INIT=0, EXEC, STOP, PROCEED, UNSPECIFIED\\

Enumération exhaustive des types d'actions possibles sur ({\em i.e}. directives reconnues par) les processus. \\

Les conséquences d'un événement sont codées (dans une donnée membre de la classe [C]Event[/C]) comme un tableau de couples {processus, action}\index{action sur un processus}. 
\begin{itemize} 
\item une action de type INIT\index{INIT} initie le processus
\item une action de type EXEC\index{EXEC} exécute complètement le processus sans possibilité de l'interrompre
\item une action de type PROCEED\index{PROCEED} poursuit l'exécution du processus jusqu'à une interruption
\item une action de type STOP\index{STOP} arrête l'exécution du processus
\end{itemize} 

L'énumération est donnée ici dans l'ordre dans lesquels les couples {processus, action} sont placés (par le moteur) dans le tableau décrivant les conséquences d'un événement particulier. Par exemple, un processus sur lequel on réalise une action de type INIT sera placé dans ce tableau avant un autre sur lequel on réalise une action de type PROCEED.\\

Quand il s'agit de plusieurs événements qu'on souhaite ordonner dans l'agenda de la simulation, l'utilisateur peut aussi utiliser ce critère (par exemple appliqué au premier processus du tableau) dans la fonction qu'il affecte par la méthode [Y]AssignBeforeInAgendaMethod[/Y]\index{AssignBeforeInAgendaMethod} à la simulation courante, et qui ordonne les événements dans l'agenda d'événements. Dans ce cas, un événement dont le premier processus reçoit une directive INIT sera placé dans l'agenda avant un autre sur lequel on réalise une action de type PROCEED.\index{ActionType}\\

La valeur UNSPECIFIED est renvoyée par [Y]FirstActionLabel[/Y] si aucune directive n'a encore été donnée.
*/
//@{
enum ActionType
  {INIT=0, EXEC, STOP, PROCEED, UNSPECIFIED};
//@}


         /**@name Classes de processus prédéfinis.
\begin{itemize}
\item READ_SEQUENTIAL_DATA_FILE_PROCESS : classe [C]ReadSequentialDataFileProcess[/C] des processus continus de lecture d'un fichier de données séquentielles 
\item INIT_FACTUAL_DATA_FILE_READING_PROCESS : classe [C]InitFactualDataFileReadingProcess[/C] des processus ponctuels d'initialisation de la lecture d'un fichier de données factuelles
\item READ_FACTUAL_DATA_RECORD_PROCESS : classe [C]ReadFactualDataRecordProcessProcess[/C] des processus ponctuels de lecture d'un enregistrement d'un fichier de données factuelles
\end{itemize}

Enumération exhaustive des symboles de classe de processus prédéfinis. \index{ProcessClassId}

Lire au chapitre 'Les fichiers', relatif à la classe [C]File[/C] et ses sous-classes, les éléments distinctifs de ces classes de processus.
*/
//@{
enum ProcessClassId {
READ_SEQUENTIAL_DATA_FILE_PROCESS,
READ_FACTUAL_DATA_RECORD_PROCESS, 
INIT_FACTUAL_DATA_FILE_READING_PROCESS
};
//@}

         /**@name Types de quantificateurs des spécifications d'ensembles d'entités.
NFIRST=1, ALLENT \index{QuantificatorId}\\

Liste exhaustive des types de quantificateurs utilisés dans les spécifications d'ensembles d'entités. \\

Une spécification d'ensembles d'entités, instance de [C]EntitySpec[/C], permet de construire une liste d'entités répondant à certains critères (voir page 'Spécifications d'ensembles d'entités'). On peut ainsi construire une liste qui comprend les {\em n} premières (avec NFIRST) ou toutes les entités (avec ALLENT) satisfaisant les critères.\label{defenumQuantificatorId}
*/
//@{
enum QuantificatorId {
   NFIRST=1     ,
   ALLENT
};
//@}

         /**@name Modes des spécifications d'ensembles d'entités.
RETURN_INSTANCES=1, INSTANTIATE \index{EntitySpecMode}\\

Liste exhaustive des modes des spécifications d'ensembles d'entités, instances de [C]EntitySpec[/C] (voir page 'Spécifications d'ensembles d'entités'). \\

 Si le mode est RETURN_INSTANCES, les entités définies par la spécification sont choisies parmi les instances existantes.\\
Si le mode est INSTANTIATE, elles sont créées au moment de l'expansion de la spécification.\label{defenumEntitySpecMode}
*/
//@{
enum EntitySpecMode {
   RETURN_INSTANCES=1     ,
   INSTANTIATE
};
//@}

         /**@name Opérateurs relationnels des spécifications de domaine.
NOT_IN=0, IN \index{RelationalOperator}\\

Liste exhaustive des opérateurs relationnels utilisés dans les spécifications de domaines de valeurs (voir page 'Spécifications de domaines de valeurs'). \\

Dans ces spécifications, les opérateurs portent, à gauche sur la valeur courante d'un descripteur et à droite sur un domaine de référence. La relation est vérifiée si la valeur courante est, respectivement, dans le domaine (IN) ou en dehors (NOT_IN).\label{defenumRelationalOperator}
*/
//@{
enum RelationalOperator {
  NOT_IN=0, IN=1
};
//@}

         //
enum OutputSpecType 
	   {EVENT_OUTPUTSPEC, P_OUTPUTSPEC, CP_OUTPUTSPEC, DP_OUTPUTSPEC, DESC_OUTPUTSPEC, ENTITY_OUTPUTSPEC};

//@} // fin Constantes enum

//----------------------------------------------------------------------------

/* *@name Couples \{processus, action\} des événements.
Le pointeur est donc accessible par une formule telle que [P]myStructProcessAction.mpProcess[/P], et le type d'action par [P]myStructProcessAction.mActionLabel[/P]\index{StructProcessAction}.
*/
// @{

class StructProcessAction
{
 public:
         // Constructeur
  StructProcessAction(Process*, ActionType);  // + JPR 991201
         ///Un pointeur sur le processus.
  Process*        mpProcess;
         ///Le type d'action sur le processus.
  ActionType      mActionType;
         ///Opérateur d'égalité. Renvoie vrai si les deux pointeurs sur les processus sont identiques, et si les deux directives sont les mêmes.
  Boolean         operator==(const StructProcessAction&) const;
         ///Opérateur d'inégalité. Renvoie vrai si les deux pointeurs sur les processus sont différents, ou si les deux directives sont différentes.
  Boolean         operator!=(const StructProcessAction&) const ;
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const StructProcessAction&) const {return TRUE;};
  Boolean         operator > (const StructProcessAction&) const {return TRUE;};
         ///Tient lieu d'opérateur de flot de sortie.
  // ci-dessous remplacement operator << par display (JPR 020715)
  // parce que la compilation avec Swig.o provoque une double définition
  // des opérateurs << de ostream et >> de istream
//friend ostream&        operator<<(ostream&, const StructProcessAction&); // + JPR 990203
 void             display() const;
};
// @}

/**@name Champ d'un enregistrement de fichier.
Il s'agit de la représentation interne d'un champ d'enregistrement de fichier\index{champ d'enregistrements de fichier} de données séquentielles. \\

La classe [C]SerialDataFile[/C] possède un attribut dont la valeur est un tableau de telles structures. Le type de cet attribut est {\em pFieldTab} (c'est-à-dire {\em Tableau<StructField*>}).\\

Lors de la lecture du fichier, l'attribut [Y]mFieldValue[/Y] de ces structures reçoit la valeur trouvée sur le support externe. Un appel tel que, par exemple, \\
[P]
\begin{verbatim}
  int val = myStructField.mFieldValue.intValue;
\end{verbatim}
[/P] 
renvoie cette valeur lorsqu'elle est supposée entière, c'est-à-dire lorsque l'autre attribut [Y]myStructField.mFieldType[/Y] de la structure est censé avoir la valeur INT. \\

Noter que la taille maximale d'une chaîne de caractères valeur d'un champ de type STRING est fixée par la variable globale [Y]gFieldMaxSize[/Y].\label{defpFieldTab}\index{StructField} \\
*/
//@{
         ///
struct StructField {
         /// La valeur du champ dans l'enregistrement.
   UnifiedValue    mFieldValue;
         /// Le type de la valeur du champ.
   ValueTypeId     mFieldType;
         // Opérateurs < et > (pour la forme).
   Boolean         operator < (const StructField&) const {return TRUE;};
   Boolean         operator > (const StructField&) const {return TRUE;};
};
//@} // fin champ

//X///////////////X/////////////////////////////X////////////////////////////
//
// Les classes d'intervalles
//
//X///////////////X/////////////////////////////X////////////////////////////

/**@name Les classes d'intervalles entre deux nombres

\begin{itemize}
\item [C]IntInterval[/C] : classe des intervalles d'entiers
\item [C]FloatInterval[/C] : classe des intervalles de flottants
\item [C]DoubleInterval[/C] : classe des intervalles de flottants double précision
\end{itemize}

On peut manipuler des intervalles d'entiers, de flottants, de flottants double précision. On construit pour cela des instances des classes [C]IntInterval[/C], [C]FloatInterval[/C], [C]DoubleInterval[/C], respectivement. \\

Les bornes inférieures et supérieures sont respectivement les membres [Y]bi[/Y] et [Y]bs[/Y], pour les trois types.\\
[P]
\begin{verbatim}
{
  ...
  IntInterval* pSpan = new IntInterval();
  pSpan.bi = 0;
  pSpan.bs = 9;
  printf("\nLes bornes de pSpan sont %d et %d.", pSpan.bi, pSpan.bs);
  ...
}
\end{verbatim}
[/P]
\begin{verbatim}
>>
Les bornes de pSpan sont 0 et 9.
\end{verbatim}
Un intervalle {\em X} est 'inférieur' (resp. 'supérieur') à un autre {\em Y} (opérateur [Y]<[/Y]) si [P]X.bi[/P] est plus petit (resp. plus grand)) que [P]Y.bi[/P].

On peut créer des intervalles en plusieurs morceaux, instances des classes [C]pIntIntervalTab[/C], [C]pFloatIntervalTab[/C], [C]pDoubleIntervalTab[/C].\\ 
[P]
\begin{verbatim}
{
  ...
  IntInterval* pSpan1 = new IntInterval();
  pSpan1.bi = 0;
  pSpan1.bs = 9;
  IntInterval* pSpan2 = new IntInterval();
  pSpan2.bi = 20;
  pSpan2.bs = 29;
  pIntIntervalTab* pSpanTab = new pIntIntervalTab();
  pSpanTab->push_back(pSpan1);
  pSpanTab->push_back(pSpan2);
  ...
}
\end{verbatim}
[/P]

*/
//@{

/* *@name Les intervalles d'entiers : IntInterval
*/
//@{
class IntInterval
{
 public:
         // Constructeur.
  IntInterval(int, int);
         // Constructeur par copie
  IntInterval(const IntInterval&);  // _no swig_
         // La borne inférieure.
  int        bi;
         // La borne supérieure.
  int        bs;
         // Opérateur d'infériorité. Renvoie {\em vrai} si {\em this} est inférieur à l'argument.
  Boolean operator < (const IntInterval&); 
         // Opérateur de supériorité. Renvoie {\em vrai} si {\em this} est supérieur à l'argument.
  Boolean operator > (const IntInterval&);
}; 
            // Le tableau de ces intervalles
typedef Tableau<IntInterval*>               pIntIntervalTab;
//@}

//X///////////////X/////////////////////////////X////////////////////////////

/* *@name Les intervalles de flottants : FloatInterval
*/
//@{
class FloatInterval
{
 public:
         // Constructeur.
  FloatInterval(float, float);
         // Constructeur par copie
  FloatInterval(const FloatInterval&);  // _no swig_
         // La borne inférieure.
  float        bi;
         // La borne supérieure.
  float        bs; 
         // Opérateur d'infériorité. Renvoie {\em vrai} si {\em this} est inférieur à l'argument.
  Boolean operator < (const FloatInterval&); 
         // Opérateur de supériorité. Renvoie {\em vrai} si {\em this} est supérieur à l'argument.
  Boolean operator > (const FloatInterval&);
}; 
            // Le tableau de ces intervalles
typedef Tableau<FloatInterval*>               pFloatIntervalTab;
//@}

//X///////////////X/////////////////////////////X////////////////////////////

/* *@name Les intervalles de flottants double précision : DoubleInterval
*/
//@{
class DoubleInterval
{   
 public:
         // Constructeur.
  DoubleInterval(double, double);
         // Constructeur par copie
  DoubleInterval(const DoubleInterval&);  // _no swig_
         // La borne inférieure.
  double        bi;
         // La borne supérieure.
  double        bs; 
         // Opérateur d'infériorité. Renvoie {\em vrai} si {\em this} est inférieur à l'argument.
  Boolean operator < (const DoubleInterval&);
         // Opérateur de supériorité. Renvoie {\em vrai} si {\em this} est supérieur à l'argument.
  Boolean operator > (const DoubleInterval&);
}; 
            // Le tableau de ces intervalles
typedef Tableau<DoubleInterval*>               pDoubleIntervalTab;
//@}

//@} // fin intervalles

//X///////////////X/////////////////////////////X////////////////////////////

// EXTERNAL REFERENCES

// extern char* ActionTypeStrings[];
// supprimé parce que la swiggisation échoue à partir de Linux 2.4.25 (Carré)
// d'où modif du code de la fonction StringActionType(ActionType) 041109

extern char gInstanceNameString[80];  // cf. CoreEvent pour la taille
extern char gErrorString[100];
extern char gUnderscore[2];
extern char gBodyLabel[5];
extern char gIterString[50];

extern int gNbGenInsert;
extern int gNbTestBefore;
extern bool gEquallyPlacedEventFlag;
extern int gFinalClock;
extern int gAnyCount;
extern int gAnyCount2;
extern int gAnyCount3;
extern int gAnyCount4;

extern int gNbProgReprise;
// extern int gMaxNbIterations; // unused (JPR 110628)
extern pMetaClassTab  gMetaClassTab;
extern UserClassIdTab gMetaClassIdTab;

//--------------------------------------------------
// symboles modifiés JPR 030417
enum GraphType {OS_POINT, OS_LINE, OS_VOIDGRAPH};
// idem  
enum AllOrEnd {OS_ALL, OS_END, OS_VOIDALLEND, };
// La bizarrerie ci-dessus (la virgule terminale) est inexplicablement
// justifiée par la docification (doc++) sous SunOS (ossau).
// Quand pas cette virgule, doc++ génère la ligne suivante :
//       \cxxitem{OS_VOIDALLEND} au lieu de \cxxitem{}
// pou l'entrée "Symboles de classe des entités du domaine."
// du fichier ../exemples/eng/UserTypes.h
//--------------------------------------------------

extern   char* majSep;   // + JPR 0002
extern   char* minSep;

enum exceptions {  // + JPR 110816
  OUT_OF_TAB_RANGE_OP, OUT_OF_TAB_RANGE_GET_ELEM, 
  OUT_OF_TAB_RANGE_GET_NTH, OUT_OF_TAB_RANGE_INSERT, 
  OUT_OF_TAB_RANGE_REMOVE, OUT_OF_TAB_RANGE_REMOVE_FROM, 
  OUT_OF_TAB_RANGE_REMOVE_TO, 
  OUT_OF_TAB_RANGE_REPLACE_L, OUT_OF_TAB_RANGE_REPLACE_I,
  MUST_BE_PRESENT_REMOVE_FIRST,
  VOID_ARRAY_FRONT, VOID_ARRAY_BACK
};

#endif  // _CORETYPES_H
