//============================================================================
// FILE: ParserSpec.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des classes utilisées dans le parseur de structures
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _PARSERSPEC_HH
#define _PARSERSPEC_HH

#include "Entity.h"

enum iteratedActionType {
  SPEC_CREATE_ENTITY=0, FIN_AJOUT_ENTITY, SPEC_AJOUT_COMP, SPEC_AJOUT_EL, SPEC_DESC_AFFECT, SPEC_ITER, SPEC_PLUG_COMP, SPEC_PLUG_EL, SPEC_BODY_AFFECT};
enum affectationMode {
  STATIC_MODE=0, DYNAMIC_MODE
};

/* // + 020430
Structures décrivant et recevant certaines informations écrites dans :\\
. le fichier externe de spécification du système à simuler\\
. le fichier des directives d'exécution. \\
Plus précisément, les informations visées portent sur les actions de construction du système incluse dans une boucle d'itération. Par exemple, lorsqu'on veut ajouter {\em n} éléments à une entité, on écrit dans le fichier externe une boucle de type 'for' incluant une action (itérative, par conséquent) de création d'un élément et d'ajout à l'entité qui doit le contenir. \\
Chaque action élémentaire dans une boucle 'for' fait l'objet d'une structure {\bf ExternEntityIteratedActionSpec} ou {\bf ExternDirectiveIteratedActionSpec}. \\
Les instances de {\bf ExternEntityIteratedActionSpec} sont rangées dans une liste, laquelle est ensuite parcourue {\em n} fois par le programme d'interprétation du fichier externe. Il en de même pour les instances de {\bf ExternDirectiveIteratedActionSpec}.\\
*/

//===========================================================================

class ExternEntityIteratedActionSpec {
 public:
         // Constructeur
   ExternEntityIteratedActionSpec();
         // Destructeur
   ~ExternEntityIteratedActionSpec();
         //
   int id;
    // comptage const/dest
   static int   getNbConst() {return _nbConst;}
   static int   getNbDest() {return _nbDest;} 

protected:
   int static   _nbConst;
   int static   _nbDest;
}; 

//----------------------------------------------
typedef 
Liste<ExternEntityIteratedActionSpec*> pExternEntityIteratedActionList;
//----------------------------------------------

class ExternEntityAdditionSpec : public ExternEntityIteratedActionSpec {
 public:
         // Constructeur
   ExternEntityAdditionSpec();
         // Destructeur
   ~ExternEntityAdditionSpec();
         // 
   char* addedEntityClassName;
         // 
   char* addedEntityInstanceName;
         // 
   bool  indexedName;
         // 
   void SetClassName(char*);
         // 
   void SetInstanceName(char*);
};  

//----------------------------------------------

class ExternEntityPlugSpec : public ExternEntityIteratedActionSpec {
 public:
         // Constructeur
   ExternEntityPlugSpec();
         // Destructeur
   ~ExternEntityPlugSpec();
         // 
   Entity* addedEntity;
}; 

//----------------------------------------------

class ExternEndEntityAdditionSpec : public ExternEntityIteratedActionSpec {
 public:
         // Constructeur
   ExternEndEntityAdditionSpec();
         // Destructeur
   ~ExternEndEntityAdditionSpec();
}; 

//----------------------------------------------

class ExternEntityDescAffectationSpec : public ExternEntityIteratedActionSpec { 
 public: 
         // Constructeur
   ExternEntityDescAffectationSpec(); 
         // Destructeur
   ~ExternEntityDescAffectationSpec();
         // 
   int mode;
         // 
   int iteratorIndex;
         // 
   ValueTypeId valueType;  
         // 
   char* addedDescriptorName;  
         //
   int   addedDescriptorIntValue;
         //
   int   addedDescriptorIntSupValue;
         //
   double addedDescriptorRealValue;
         //
   char* addedDescriptorStringValue;
         //
   Entity* addedDescriptorEntityValue;
         //
   char* addedDescriptorEntityValueClassName; 
         //
   EntitySpec* addedDescriptorEntitySpecValue; 
         //
   void  SetDescriptorName(char*);
         //
   void  SetStringValue(char*);
}; 

//----------------------------------------------

class ExternBodyAffectationSpec : public ExternEntityIteratedActionSpec { 
 public: 
         // Constructeur
   ExternBodyAffectationSpec(); 
         // Destructeur
   ~ExternBodyAffectationSpec();
         // 
   ValueTypeId valueType;  
         // 
   char* addedMethodName;

   void        (*addedVoidMethodBody)     (EntityMethod*);
   bool        (*addedPredicateBody)      (EntityMethod*);
   int         (*addedIntMethodBody)      (EntityMethod*);
   float       (*addedFloatMethodBody)    (EntityMethod*);
   double      (*addedDoubleMethodBody)   (EntityMethod*);
   Entity*     (*addedEntityMethodBody)   (EntityMethod*);
   char*       (*addedStringMethodBody)   (EntityMethod*);
   pEntityTab* (*addedEntityTabMethodBody)(EntityMethod*);
         //
   void  SetMethodName(char*);
};  

//----------------------------------------------

class ExternEntityIterationSpec : public ExternEntityIteratedActionSpec {
 public: 
         // Constructeur
   ExternEntityIterationSpec(int, int, int, int);  
         // Destructeur
   ~ExternEntityIterationSpec(); 
         // 
   int   level;  
         // 
   int   iteratorIndex;   
         //
   int   iteratorBegin;   
         //
   int   iteratorEnd;
         //
   pExternEntityIteratedActionList* pActionList;
         // 
   void  addAction(ExternEntityIteratedActionSpec*);
         //
   void  execActionList(pEntityList*,Tableau<int>*);
};

typedef 
Liste<ExternEntityIterationSpec*> pExternEntityIterationList; 

//===========================================================================

class ExternDirectiveIteratedActionSpec {
 public:
         // Constructeur
   ExternDirectiveIteratedActionSpec();
         // Destructeur
   ~ExternDirectiveIteratedActionSpec();
         //
   int id;
    // comptage const/dest
   static int   getNbConst() {return _nbConst;}
   static int   getNbDest() {return _nbDest;} 

protected:
   int static   _nbConst;
   int static   _nbDest;
};  

//----------------------------------------------
typedef 
Liste<ExternDirectiveIteratedActionSpec*> pExternDirectiveIteratedActionList;  

//----------------------------------------------

class ExternDirectiveIterationSpec : public ExternDirectiveIteratedActionSpec {
 public: 
         // Constructeur
   ExternDirectiveIterationSpec(int, int, int, int);  
         // Destructeur
   ~ExternDirectiveIterationSpec(); 
         // 
   int   level;  
         // 
   int   iteratorIndex;   
         //
   int   iteratorBegin;   
         //
   int   iteratorEnd;
         //
   pExternDirectiveIteratedActionList* pActionList;
         // 
   void  addAction(ExternDirectiveIteratedActionSpec*);
         //
   void  execActionList(Tableau<int>*);  // un seul arg
};

typedef 
Liste<ExternDirectiveIterationSpec*> pExternDirectiveIterationList;

//===========================================================================


#endif  // _PARSERSPEC_HH
