//============================================================================
// FILE: Tableau.h 
//
// TYPE: C++TYPEDEF
// DESCRIPTION: définition de la classe générique Tableau
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _TABLEAU_H
#define _TABLEAU_H

// PROJECT INCLUDES

// SYSTEM INCLUDES
#include <stdio.h>  // pour FILE

// PROJECT INCLUDES
class BasicEntity; // pour extract_first...

/*
Un {\bf Tableau}\index{Tableau} est une structure de donnée qui permet de déclarer un tableau sans mention de sa taille. Comme les "double ended queue", elle supporte l'accès aléatoire (opérateur {\bf []}) et les opérations d'insertion et de retrait aux deux extrémités et au milieu du tableau. Les opérations d'insertion et de retrait en t\^ete ne sont pas disponibles pour les "vecteurs".\\

Un aperçu de la comparaison avec les classes {\bf deque}\index{deque} et {\bf vector}\index{vector} de la librairie STL est donné ci-dessous. Les structures sont initialisées par :
 
\begin{verbatim}
    deque<int> tab;
//  vector<int> tab;
//  Tableau<int> tab;
    for (int i=0; i<10; i++) tab.push_back(i);
\end{verbatim}

\begin{verbatim}
initialisation avec   513 push_front()             deque   : 0'00''01
initialisation avec   514 push_front()             deque   : array bounds write
initialisation avec push_front()                   vector  : non disponible 
initialisation avec   513 push_front()             Tableau : 0'00''06 
initialisation avec 10000 push_front()             Tableau : 0'21''12

initialisation avec   513 insert(begin())          deque   : 0'00''41
initialisation avec   514 insert(begin())          deque   : array bounds write
initialisation avec 10000 insert(begin())          vector  : 0'14''86
initialisation avec 10000 insert(0)                Tableau : 0'21''34

initialisation avec 10000 push_back()              deque   : 0'00''04
initialisation avec 10000 push_back()              vector  : 0'00''03
initialisation avec 10000 push_back()              Tableau : 0'21''94

initialisation avec 10000 push_back(), puis : 
10000 pop_front()                                  deque   : 0'00''03
10000 pop_front()                                  vector  : non disponible 
10000 pop_front()                                  Tableau : 0'20''94

initialisation avec 10000 push_back(), puis : 
10000 erase(begin())                               deque   : 0'00''17
10000 erase(begin())                               vector  : 0'12''34
10000 remove(0)                                    Tableau : 0'20''67

initialisation avec 10000 push_back(), puis : 
10000 pop_back()                                   deque   : 0'00''03
10000 pop_back()                                   vector  : 0'00''01
10000 pop_back()                                   Tableau : 0'20''67

10000000 front()                                   deque   : 0'10''32
10000000 front()                                   vector  : 0'05''96 
10000000 front()                                   Tableau : 0'05''28

10000000 back()                                    deque   : 1'33''80
10000000 back()                                    vector  : 0'05''90
10000000 back()                                    Tableau : 0'06''37

10000000 size()                                    deque   : 0'03''93
10000000 size()                                    vector  : 0'08''43
10000000 size()                                    Tableau : 0'04''30

10000000 empty()                                   deque   : 0'04''26
10000000 empty()                                   vector  : 0'08''44
10000000 empty()                                   Tableau : 0'05''14

1000000 {push_front(99) + pop_front()}             deque   : 0'04''78
1000000 {push_front(99) + pop_front()}             vector  : non disponible
1000000 {insert(begin(), 99) + erase(begin())}     vector  : 0'09''77
1000000 {push_front(99) + pop_front()}             Tableau : 0'17''80

1000000 {push_back(99) + pop_back()}               deque   : 0'03''79
1000000 {push_back(99) + pop_back()}               vector  : 0'01''26
1000000 {push_back(99) + pop_back()}               Tableau : 0'18''21

500000 {insert(begin(), 99) + erase(begin())}      deque   : 0'10''98
500000 {insert(begin(), 99) + erase(begin())}      vector  : 0'04''88
500000 {insert(0, 99) + remove(0) }                Tableau : 0'09''26

500000 {insert(begin()+5, 99) + erase(begin()+5)}  deque   : 0'40''56
500000 {insert(begin()+5, 99) + erase(begin()+5)}  vector  : 0'03''65
500000 {insert(5, 99) + remove(5)}                 Tableau : 0'09''22

   513 {insert(begin()+6, 99) + erase(begin()+6)}  deque   : 0'00''05
   514 {insert(begin()+6, 99) + erase(begin()+6)}  deque   : Segmentation fault
   // si deque<double>, le Segmentation fault arrive dès 256
500000 {insert(begin()+6, 99) + erase(begin()+6)}  deque   : Segmentation fault
500000 {insert(begin()+6, 99) + erase(begin()+6)}  vector  : 0'03''45
500000 {insert(6, 99) + remove(6))}                Tableau : 0'09''33

   513 {insert(begin()+9, 99) + erase(begin()+9)}  deque   : 0'00''05
   514 {insert(begin()+9, 99) + erase(begin()+9)}  deque   : Segmentation fault
   // si deque<double>, le Segmentation fault arrive dès 256
500000 {insert(begin()+9, 99) + erase(begin()+9)}  deque   : Segmentation fault
500000 {insert(begin()+9, 99) + erase(begin()+9)}  vector  : 0'02''78
500000 {insert(9, 99) + remove(9)}                 Tableau : 0'09''65

500000 {insert(begin()+10, 99) + erase(begin()+10)} deque  : 0'28''21
500000 {insert(begin()+10, 99) + erase(begin()+10)} vector : 0'01''41
500000 {insert(9, 99) + remove(9)}                 Tableau : 0'09''16

     1 {insert(begin()+11, 99) + erase(begin()+11)} deque  : Segmentation fault
     1 {insert(begin()+11, 99) + erase(begin()+11)} vector : Segmentation fault
     1 {insert(11, 99) + remove(9)}                Tableau : index out of range

10000000 [0]                                       deque   : 0'43''89
10000000 [0]                                       vector  : 0'06''69
10000000 [0]                                       Tableau : 0'06''88

10000000 [5]                                       deque   : 0'44''05
10000000 [5]                                       vector  : 0'06''72
10000000 [5]                                       Tableau : 0'06''78

10000000 [9]                                       deque   : 0'43''00
10000000 [9]                                       vector  : 0'06''61
10000000 [9]                                       Tableau : 0'06''86

10000000 [10]                                      deque   : 0'43''01
10000000 [10]                                      vector  : 0'06''54
10000000 [10]                                      Tableau : index out of range

100000 {10 push_back(i) + erase(begin(), end())}   deque   : 0'09''36
100000 {10 push_back(i) + erase(begin(), end())}   vector  : 0'01''00
100000 {10 push_back(i) + erase()}                 Tableau : 0'14''64
\end{verbatim}
*/

/**@name CDPFORTEXStructures de données génériques : les tableaux et les listes */
//@{ // fermeture dans Liste.h

/**@name La classe générique des tableaux 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include "Tableau.h"
[/P]
\end{itemize}   

Contrairement à la structure de [C]Liste[/C], un [C]Tableau[/C] est une structure de donnée à accès indexé. Elle peut être vide.\\

Un tableau supporte l'ajout d'un élément de rang {\em 0} s'il est vide, l'ajout d'un élément avant le premier élément ou après le dernier, et le retrait d'un élément existant. On peut accéder à la valeur d'un élément désigné par son indice, et la modifier.
*/
//@{  // classe + fonctions sur Tableau

/**@name La classe : template <class T> class Tableau*/
//@{
  /// 
template <class T> 
class Tableau
{
public:

/**@name LIFECYCLE
\label{constDestTableau}
\\
Une trace\index{trace (Tableau : création/destruction)} de la création et de la destruction des tableaux peut \^etre obtenue en donnant auparavant la valeur {\small {\tt TRACE_TABLEAU_ON}} à la variable [Y]gTraceTableauMode[/Y]\index{gTraceTableauMode}. Sinon, la trace doit \^etre inhibée avec la valeur {\small {\tt TRACE_TABLEAU_OFF}}, quelque part dans le programme. Pour les utilisateurs de DIESE, cette initialisation est réalisée dans [Y]CoreTypes.o[/Y]}.
\begin{verbatim}
[P]int gNbConstTableau = 0;
int gNbDestTableau = 0; 
boolTraceTableauMode gTraceTableauMode =TRACE_TABLEAU_ON;

int main (...) { 
 ...
}[/P]
\end{verbatim}

Les constructeurs et le destructeur incrémentent et décrémentent le nombre de tableaux créés et détruits, respectivement dans les variables globales [Y]gNbConstTableau[/Y]\index{gNbConstTableau} et [Y]gNbDestTableau[/Y]\index{gNbDestTableau}. Ces deux variables doivent être initialisées à 0 quelque part dans le programme de l'utilisateur.\\

Lors d'un ajout ou un retrait d'élément, la mémoire allouée au tableau augmente ou diminue. Par défaut de spécification contraire, ce changement est effectué à chaque ajout ou retrait. Il est possible d'effectuer par palliers ces changements de taille allouée : la largeur du pallier est l'incrément, argument de certains constructeurs. Par exemple, si l'incrément est 10, la première allocation réserve la mémoire pour 10 éléments, et on ne fera une nouvelle réservation que si on ajoute un élément de rang 10 ou au-delà.
*/
//@{
          /// Constructeur de base d'un tableau de taille nulle, avec incrémentation de taille par défaut (1).
   Tableau();
          /// Constructeur de base d'un tableau de taille nulle, avec incrémentation de taille spécifiée.
   Tableau(int increment);
          /// Constructeur d'un tableau de n éléments identiques à T, avec incrémentation de taille par défaut (1). 
   Tableau(T, int n); 
          /// Constructeur d'un tableau de n éléments identiques à T, avec incrémentation de taille spécifiée. 
   Tableau(T, int n, int increment);                                
          /// Constructeur par copie.
   Tableau(const Tableau<T>&); 
          /// Destructeur. Ne détruit pas les éléments, sauf s'il s'agit d'un tableau d'intervalles.               
   ~Tableau(void);
 
          ///  Affecte le tableau passé en argument au tableau récepteur du message. Les deux tableaux doivent avoir la m\^eme taille. Les deux tableaux sont différents mais ont le même contenu après l'opération. \index{Tableau_m!operation!copy}
   const Tableau<T>&   copy_from (const Tableau<T>&);

//@}             

/**@name OPERATORS
\\

De façon générale, lorsque le prérequis d'un opérateur ou d'une opération n'est pas satisfait, le programme appelant est arr\^eté. L'adresse du tableau pour lequel l'opération a échoué est affichée. L'utilisateur peut utiliser la variable globale [Y]gTraceTableauMode[/Y]\index{gTraceTableauMode} pour avoir la trace de la création du tableau.
*/
//@{         
          ///  Renvoie TRUE si le tableau à droite a la m\^eme taille que celui de gauche et contient les mêmes valeurs dans le m\^eme ordre, FALSE sinon.\index{Tableau_m!operator!==}
   int            operator == (const Tableau<T>&) const; 

          ///  Assigne le tableau à droite au tableau à gauche. Les deux tableaux peuvent ne pas avoir la même taille. Les deux tableaux sont différents mais ont le même contenu après l'opération. \index{Tableau_m!operator!=}
   const Tableau<T>&     
                   operator = (const Tableau<T>&);

          ///  Renvoie une référence sur l'élément de position k dans le tableau. k doit \^etre compris entre 0 et la taille du tableau moins 1.\index{Tableau_m!operator![]} 
   T&              operator [] (int k) const;  
//@}             

/**@name [SC]Propriétés du tableau[/SC] */
//@{      
          ///  Renvoie le nombre d'éléments du tableau.\index{Tableau_m!access!size} 
   int            size() const;
          //   Modifie l'attribut nombre d'éléments (de type T) du tableauRenvoie le nombre d'éléments du tableau.
   void           size(int);
          ///  Renvoie le pas d'incrémentation de la taille de la mémoire allouée au tableau.\index{Tableau_m!access!increment} 
   int            sizeIncrement() const;
          ///  Etablit le pas d'incrémentation (strictement positif) de la taille de la mémoire allouée au tableau. 
   void           sizeIncrement(int); // + JPR 071017
          ///  Renvoie TRUE si le tableau est vide, FALSE sinon.\index{Tableau_m!access!empty} 
   int            empty() const; 
//@}             

/**@name [SC]Accès aux éléments du tableau[/SC] */
//@{  
          ///  Renvoie le premier élément du tableau (celui d'indice 0). Le tableau ne doit pas \^etre vide.\index{Tableau_m!access!front}    
   T&              front() const; 
          //  Renvoie l'élément de position k dans le tableau. k doit \^etre compris entre 0 et la taille du tableau moins 1. 
   T&              get_element(int k) const;  // caché (reste visible : get_nth)
          ///  Renvoie l'élément de position k dans le tableau. k doit \^etre compris entre 0 et la taille du tableau moins 1.\index{Tableau_m!access!get_nth} 
   T&              get_nth(int k) const; 
   /*  - JPR 110816
          //  Renvoie l'élément de position k dans le tableau. k doit \^etre compris entre 0 et la taille du tableau moins 1.
   T&              get_element2(int k, char*) const;   // + JPR 090414
          //  Renvoie l'élément de position k dans le tableau. k doit \^etre compris entre 0 et la taille du tableau moins 1.
   T&              get_nth2(int k, char*) const;   // + JPR 090414  
   */
          ///  Renvoie le dernier élément du tableau. Le tableau ne doit pas \^etre vide.\index{Tableau_m!access!back}    
   T&              back() const;  
          ///  Renvoie -9 si l'élément n'est pas dans le tableau, et l'indice de sa première occurence sinon.\index{Tableau_m!access!position}
   int             position(const T & element) const;
        /// Renvoie le premier élément du tableau d'entités tel que la valeur du descripteur dont le symbole est passé en premier argument soit la valeur entière passée en second argument. \index{Tableau_m!operation!extract_first_with_int_value}
   T               extract_first_with_int_value(int, int) const;
        /// Renvoie le premier élément du tableau d'entités tel que la valeur du descripteur dont le symbole est passé en premier argument soit la valeur du pointeur sur une entité passé en second argument. \index{Tableau_m!operation!extract_first_with_entity_value}
   T               extract_first_with_entity_value(int, BasicEntity*) const;
        /// Renvoie le tableau des éléments dont le symbole de classe est passé en argument, extraits du tableau d'entités récepteur de ce message. \index{Tableau_m!operation!extract_class_named_entities}
   Tableau<T>*               extract_class_named_entities(int) const;
//@}             

/**@name [SC]Comparaisons des contenus de deux tableaux[/SC] */
//@{ 
          ///  Renvoie {\em vrai} si le récepteur du message et le tableau passé en argument contiennent contiennent au moins un élément commun, et {\em faux} sinon.\index{Tableau_m!access!commonElement}
bool               commonElement(const Tableau<T> & tableau);
          ///  Renvoie {\em vrai} si le récepteur du message et le tableau passé en argument contiennent les mêmes éléments, compte non tenu de la position des éléments, et {\em faux} sinon.\index{Tableau_m!access!sameElements}
bool               sameElements(const Tableau<T> & tableau);

          ///  Renvoie {\em vrai} si le récepteur du message contient tous les éléments du tableau passé en argument, et {\em faux} sinon.
bool               containsAllElements(const Tableau<T> & tableau);
          ///  Renvoie {\em vrai} si le récepteur du message et le tableau passé en argument contiennent les mêmes {\em n} premiers éléments dans le même ordre, et {\em faux} sinon.\index{Tableau_m!access!sameFirstElements}
          
bool               sameFirstElements(const Tableau<T> & arg, int n);
          ///  Renvoie le tableau des éléments communs en fin des deux tableaux.
Tableau<T>*         commonTail(const Tableau<T> & arg);
//@}  
          //  Renvoie la représentation interne des éléments du tableau.
   T*             content() const;
          //  Modifie la représentation interne des éléments du tableau. 
   void           content(T*);

/**@name [SC]Ajout, retrait, insertion d'éléménts[/SC] */ 
//@{
          ///  Insère un élément en tête du tableau (son indice sera 0). Le nombre d'éléments est augmenté de 1.\index{Tableau_m!operation!push_front} 
   void            push_front(const T &);
          ///  Insère un élément dans le tableau, en position donnée en premier argument. Le nombre d'éléments est augmenté de 1. k doit \^etre compris entre 0 et la taille du tableau avant l'insertion.\index{Tableau_m!operation!insert} 
   void            insert(int k, const T & element);  
          ///  Ajoute un élément au tableau, après le dernier. Le nombre d'éléments est augmenté de 1.\index{Tableau_m!operation!push_back(const T \&)}
   void            push_back(const T & element);
          ///  Ajoute un élément en queue du tableau s'il n'y est pas déjà quelque part. Renvoie 1 ({\em vrai}) si l'ajout a été fait et 0 sinon.\index{Tableau_m!operation!push_back_ifnew(const T \&)}
   int             push_back_ifnew(const T & element);
   int             push_back_ifnew(const T & element, int bidon);
          ///  Ajoute après le dernier élément du tableau les éléments du tableau passé en argument. Le nombre d'éléments est augmenté de la taille de ce dernier.\index{Tableau_m!operation!push_back(const Tableau<T> \&)}
   void            push_back(const Tableau<T> & tableau);
          ///  Ajoute après le dernier élément du tableau les éléments du tableau passé en argument, s'ils ne sont pas déjà quelque part dans le tableau. Renvoie 1 ({\em vrai}) si un élement au moins a été ajouté, et 0 sinon.\index{Tableau_m!operation!push_back_ifnew(const Tableau<T> \&)}
   int             push_back_ifnew(const Tableau<T> & tableau); 
          ///  Retire le premier élément du tableau (celui d'indice 0). Le tableau ne doit pas \^etre vide. Son nombre d'éléments est diminué de 1.\index{Tableau_m!operation!pop_front}
   void            pop_front();
          ///  Retire du tableau l'élément d'indice passé en argument. Le nombre d'éléments est diminué de 1. {\em k} doit \^etre compris entre 0 et {\em n-1}, où {\em n} est la taille du tableau avant le retrait.\index{Tableau_m!operation!remove}
   void            remove(int k);
           ///  Retire du tableau les éléments d'indice supérieur ou égal à l'argument {\em k}. Le nombre d'éléments est diminué de {\em n-k}. {\em k} doit \^etre compris entre 0 et {\em n}, où {\em n} est la taille du tableau avant le retrait. L'appel {\tt remove_from(0)} a le même effet que l'appel {\tt erase()}. L'appel {\tt remove_from(n-1)} a le même effet que {\tt pop_back()}. L'appel {\tt remove_from(n-1)} n'a pas d'effet.\index{Tableau_m!operation!remove_from}
   void            remove_from(int k);
           ///  Retire du tableau les éléments d'indice inférieur à l'argument {\em k}. Le nombre d'éléments est diminué de {\em k}. {\em k} doit \^etre compris entre 0 et {\em n}, où {\em n} est la taille du tableau avant le retrait. L'appel {\tt remove_to(0)} n'a pas d'effet. L'appel {\tt remove_to(1)} a le même effet que {\tt pop_front()}. L'appel {\tt remove_to(n)} a le même effet que {\tt erase()}.\index{Tableau_m!operation!remove_to}
   void            remove_to(int k); 
          ///  Retire du tableau tous les éléments identiques à celui passé en argument. Le nombre d'éléments est diminué du nombre d'occurences.\index{Tableau_m!operation!remove_all} 
   void            remove_all(const T & element); 
          ///  Retire du tableau le premier élément identique à celui passé en argument. L'argument doit effectivement être présent dans le tableau, si le second argument est TRUE. Le nombre d'éléments est alors diminué de 1.
   void            remove_first(const T & element, bool mustBePresent); 
          ///  Retire du tableau le premier élément identique à celui passé en argument. L'argument doit effectivement être présent dans le tableau. Le nombre d'éléments est alors diminué de 1.\index{Tableau_m!operation!remove_first} 
   void            remove_first(const T & element);  
          ///  Retire le dernier élément du tableau. Le tableau ne doit pas \^etre vide. Son nombre d'éléments est diminué de 1.\index{Tableau_m!operation!pop_back}
   void            pop_back();
          ///  Retire tous les éléments du tableau, s'il en a. Sa taille devient nulle. Son incrément de taille est 1.\index{Tableau_m!operation!erase}
   void            erase();
          ///  Retire tous les éléments du tableau, s'il en a. Sa taille devient nulle. Son incrément de taille est spécifié.\index{Tableau_m!operation!erase}
   void            erase(int);  
          ///  Remplace l'élément d'indice k dans le tableau par l'élément donné en second argument. La taille du tableau est inchangée. "X.replace_by_item(k, e);" est équivalent à "X[k] = e";\index{Tableau_m!operation!replace_by_item}
   void            replace_by_item(int k, const T & element); 
          ///  Remplace l'élément d'indice k dans le tableau par les éléments du tableau donné en second argument.\index{Tableau_m!operation!replace_by_list}
   void            replace_by_list(int k, const Tableau<T> & tableau); 

          ///  Affecte le deuxième argument comme valeur de l'élément d'indice donné en premier argument. Si l'indice pointe au-delà du dernier élément, la taille du tableau est augmentée en conséquence. L'appel "X.set_nth(k, e)" n'est donc pas équivalent à "X[k] = e".\index{Tableau_m!operation!set_nth}
   void            set_nth(int k, const T& element);
//@}  

/**@name [SC]Affichage[/SC] */ 
//@{

         /// Affiche le contenu du tableau. Ne saute de ligne ni avant ni après.\index{Tableau_m!operation!display}
   void            display() const;
         // 
   void            display(FILE*) const;

        /// Affiche le contenu d'un tableau d'intervalles (et seulement d'intervalles), avec les valeurs des bornes. L'argument est affiché en titre.\index{Tableau_m!operation!display_bounds}
   void            display_bounds(char*);

        /// Affiche le contenu d'un tableau d'entités (et seulement d'entités), avec les noms d'instances. \index{Tableau_m!operation!display_names}
   void            display_names() const;
//@}  

/**@name [SC]Tri des éléments[/SC] */ 
//@{
        //
   void            insert_sort(int lb, int ub);
        //
   void            insert_sort_inverse(int lb, int ub);
        /// Trie le tableau dans l'ordre croissant des éléments. La fonction d'ordre est l'opérateur '<', éventuellement surchargé. \index{Tableau_m!quick_sort}
   void            quick_sort(int lb, int ub);
        /// Trie le tableau dans l'ordre décroissant des éléments. La fonction d'ordre est l'opérateur '>', éventuellement surchargé. \index{Tableau_m!quick_sort_inverse}
   void            quick_sort_inverse(int lb, int ub);

        // + JPR 080728 pour SISOL
   void            insert_sort(int lb, int ub, int (*comparator)(const T e1, const T e2));
        /// Trie le tableau dans l'ordre spécifié par la fonction passée en troisième argument. Cette fonction est supposée renvoyer respectivement -1, 0 ou 1 selon que le premier argument est inférieur, égal ou supérieur au second.
   void            quick_sort(int lb, int ub, int (*comparator)(const T e1, const T e2));

        // Le tableau doit être déjà trié par quick_sort.
   void            reduce_to_disjunction();

        // 
   Tableau<Tableau<T>*>*            split();
//@}  

//@}    
protected :    

   int        nbElements;
   int        increment;
   T*         contenu;

}; // _end class_

//@} // fin classe

//X///////////////////////////////////////////////////////////////////////////
//
// Fonctions diverses
//
//X///////////////////////////////////////////////////////////////////////////  

/**@name Fonctions diverses sur les tableaux 
\\
Ces fonctions ont un effet équivalent certaines méthodes de la classe [C]Tableau[/C] : au lieu d'adresser un message à un tableau, on passe ce tableau en argument de la fonction. Par exemple, les deux séquences suivantes sont équivalentes :
\begin{verbatim}
[P]Tableau<int>* pTab = new Tableau<int>();
pTab->set_nth(5, -9);[/P]
\end{verbatim}
et
\begin{verbatim}
[P]Tableau<int>* pTab = new Tableau<int>();
set_nth_num(*pTab, 5, -9);[/P]
\end{verbatim}

*/
//@{  

/* Ajoute le tableau pTab (plus exactement l'adresse du tableau) en fin du tableau tabTab (tableau d'adresses de tableaux)\ldots
 et renvoie 1 ({\em vrai}), s'il n'y est pas déjà, au sens où pTab contiendrait les mêmes éléments (sans tenir compte de la position) qu'un tableau pointé dans tabTab.\index{Tableau_f!push_tab_back_ifnotpresent} */
/* !!! ligne trop longue pour 'tar xf' de la release sous Cygwin 
   si la doc ci-dessus est docisée et la déclaration ci-dessous commentée
   OK si pas de docisation et décl. commentée
   OK si docisation et décl. décommentée */
//template <class T>
//int               push_tab_back_ifnotpresent(Tableau<Tableau<T>*> & tabTab, 
//					     Tableau<T>*    pTab);

/* Ajoute le tableau pTab (plus exactement l'adresse du tableau) en queue du tableau tabTab (tableau d'adresses de tableaux).\index{Tableau_f!push_tab_back} */
//template <class T>
//void              push_tab_back(Tableau<Tableau<T>*> & tabTab, 
//				Tableau<T>*    pTab);


/** Attribue la valeur {\em pObj} à l'élément d'indice {\em k}. 
{\small {\tt pTab}} est un tableau de pointeur sur des instances de [C]BasicEntity[/C]. {\small {\tt pObj}} est un pointeur sur une instance de [C]BasicEntity[/C]. */
template <class T>
 void             set_nth_obj(Tableau<T>& pTab, int k, T pObj);

/** Attribue la valeur {\em val} à l'élément d'indice {\em k}. 
{\small {\tt val}} est un nombre entier ou flottant en double précision. */
template <class T> 
void              set_nth_num(Tableau<T>&, int, const T& val);
template <class T> 
void              set_nth_numm(Tableau<T>&, int, const T& val);

/** Etablit la valeur de l'élément du tableau à double entrée tab, celui d'indice "ligne" i et d'indice "colonne" j.\index{Tableau_f!set_nth_elem} N'est défini que pour le type {\em int}.
*/
template <class T>
void              set_nth_elem(Tableau<Tableau<T>*>& tab, 
			       int i, int j, T); 

/** Ajoute une valeur (si non encore présente) à la "ligne" i du tableau à double entrée tab.\index{Tableau_f!push_back_in_row} N'est défini que pour le type {\em int}. */
template <class T>
void              push_back_in_row(Tableau<Tableau<T>*>& tab, 
			           int i, T);

/** Renvoie un élément du tableau à double entrée tab, celui d'indice "ligne" i et d'indice "colonne" j.\index{Tableau_f!get_nth_elem} N'est défini que pour le type {\em int}. */
template <class T>
T                 get_nth_elem(Tableau<Tableau<T>*>& tab, 
							   int i, int j);

/* * Construit et renvoie le tableau des {\em n} premiers éléments de {\em tab}.
Si {\em n} est inférieur ou égal à {\em 0}, on construit et renvoie un tableau vide. Si {\em tab} est vide ou si {\em n} est plus grand que la taille de {\em tab}, le tableau renvoyé est une copie de {\em tab}. \index{Tableau_f!get_nfirst_elem} */
template <class T>
Tableau<T>*       get_nfirst_elem(Tableau<T>& tab, 
								  int n);   // non templatisé
 

        //
   void           insert_sort(int lb, int ub);
   void           insert_sort_inverse(int lb, int ub);
        //
   void           quick_sort(int lb, int ub);
   void           quick_sort_inverse(int lb, int ub);

class BasicEntity;

/** Applique l'opérateur {\em delete} aux sous-tableaux et au tableau lui-m\^eme. N'est défini que pour les types {\em int}, {\em Entity*} et {\em BasicEntity*}. */
template <class T>
   void           depth_delete(Tableau<Tableau<T>*>& tab);

/** Affiche le contenu du tableau à deux niveaux, et deux seulement, passé en argument.  N'est défini que pour les types {\em int}, {\em Entity*} et {\em BasicEntity*}.\index{Tableau_f!depth_display} */
template <class T>
   void           depth_display(Tableau<Tableau<T>*>& tab);

/** Affiche les noms d'instances du contenu du tableau à deux niveaux, et deux seulement, passé en argument. N'est défini que pour les types {\em Entity*} et {\em BasicEntity*}.\index{Tableau_f!depth_display_names} */
template <class T> 
   void           depth_display_names(Tableau<Tableau<T>*>& tab);


//@}  // fin fonctions

//============================================================================
//
// DESCRIPTION: Fonctions de produits cartésiens
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//
//=X===============X=============================X============================

/**@name Fonctions générant des produits cartésiens */
//@{

         /// Produit cartésien de deux tableaux. Les arguments sont des tableaux du même type. \newlineEn entrée : TI : (a b)  TJ : (x y) \newlineEn sortie : ((a x) (a y) (b x) (b y))

template <class T>
Tableau<Tableau<T>*> Combine_11(Tableau<T>& TI, 
								Tableau<T>& TJ);
template <class T>
Tableau<Tableau<T>*>* combine_11(Tableau<T>* TI, 
								 Tableau<T>* TJ);
//---------------------------------------------------------------------------- 
// Description: produit cartésien de deux tableaux de même type :
//  TI : (a b)  TJ : (x y) 
//  resultat : ((a x) (a y) (b x) (b y))
// Precondition:
// RETURN: 
//----------------------------------------------------------------------------


         /// Produit cartésien d'un tableau par un tableau de tableaux. Le premier argument est un tableau. Le second argument est tableaux de tableaux du même type. \newlineEn entrée : TI : (a u)  TTJ : ((x y) (w z)) \newlineEn sortie : ((a x y) (a w z) (u x y) (u w z))

template <class T>
Tableau<Tableau<T>*> Combine_1n(Tableau<T>& TI, 
								Tableau<Tableau<T>*>& TTJ);
template <class T>
Tableau<Tableau<T>*>* combine_1n(Tableau<T>* TI, 
								 Tableau<Tableau<T>*>* TTJ);
//---------------------------------------------------------------------------- 
// Description: produit cartésien "appendé" de deux tableaux
//              le premier argument est un tableau        
//              le second argument est tableaux de tableaux du même type
//  TI  :  (a u)  
//  TTJ : ((x y) (w z)) 
//  resultat: ((a x y) (a w z) (u x y) (u w z))
// Precondition:
// RETURN: 
//----------------------------------------------------------------------------


         /// Produit cartésien d'un tableau de tableaux par un tableau. Le premier argument est un tableau de tableaux du même type que le second argument. \newlineEn entrée : TTI : ((a t) (a u))  TJ : (x y) \newlineEn sortie : ((a t x) (a t y) (a u x) (a u y))

template <class T>
Tableau<Tableau<T>*> Combine_n1(Tableau<Tableau<T>*>& TTI, 
								Tableau<T>& TJ);
template <class T>
Tableau<Tableau<T>*>* combine_n1(Tableau<Tableau<T>*>* TTI, 
								 Tableau<T>* TJ);
//---------------------------------------------------------------------------- 
// Description: produit cartésien "appendé" de deux tableaux
//              le premier argument est un tableau de tableaux du même type
//              que le second argument
//  TTI : ((a t) (a u) (b t) (b u))  TJ : (x y) 
//  resultat: ((a t x) (a t y) (a u x) (a u y) (b t x) (b t y) (b u x) (b u y))
// Precondition:
// RETURN: 
//----------------------------------------------------------------------------


         /// Produit cartésien de deux tableaux de tableaux. Les arguments sont des tableaux de tableaux du même type. \newlineEn entrée : TTI : ((a t) (a u))  TTJ : ((x y) (w z)) \newlineEn sortie : ((a t x y) (a t w z) (a u x y) (a u w z))

template <class T>
Tableau<Tableau<T>*>    Combine_nn(Tableau<Tableau<T>*>& TTI, 
								   Tableau<Tableau<T>*>& TTJ);
template <class T>
Tableau<Tableau<T>*>*    combine_nn(Tableau<Tableau<T>*>* TTI, 
									Tableau<Tableau<T>*>* TTJ);
//---------------------------------------------------------------------------- 
// Description: produit cartésien "appendé" de deux tableaux
//              les arguments sont des tableaux de tableaux du même type
//  TTI : ((a t) (a u))  TJ : ((x y) (w z)) 
//  resultat: ((a t x y) (a t w z) (a u x y) (a u w z))
// Precondition:
// RETURN: 
//----------------------------------------------------------------------------

//@}  // fin Fonctions de produits cartésiens

template <class T>  // + JPR 110816
void                TabIndexOutOfRange (int exception, char* leService, 
										Tableau<T>* TTI, int index, 
										bool exit);
template <class T>  // + JPR 110816
void                TabIndexOutOfRange (int exception, char* leService, 
										const Tableau<T>* TTI, int index, 
										bool exit);

template <class T>  // + JPR 110816
void                VoidArray(int exception, char* service,
							  Tableau<T>* TTI, bool flag); 
template <class T>  // + JPR 110816
void                VoidArray(int exception, char* service,
							  const Tableau<T>* TTI, bool flag); 

template <class T>  // + JPR 110816
void                MustBePresent(int exception, char* service,
								  Tableau<T>* TTI, bool flag, bool display); 

// suite dans Liste.h (avec fermeture de "Structures de données génériques : T & L"

#endif  //_TABLEAU_H
