//============================================================================
// FILE: C_CoreCriterion.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des critères particuliers à CD
// 
// AUTHORS: Slougui (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_CORECRITERION_HH
#define _C_CORECRITERION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"
#include "C_Criterion.h"

 
/**@name Les critères spécialisés (prédéfinis) du gestionnaire du système
\label{classeCriterion}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_CoreCriterion.h} \end{itemize}
*/
//@{

//----------------------------------------------------------------------------
// GLOBAL_TARGET_CRITERION
//----------------------------------------------------------------------------
/**@name Le critère d'atteinte des cibles */
//@{
          ///
class GlobalTargetCriterion : public Criterion 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  GlobalTargetCriterion();  // _no swig_
//@}

protected :

}; // _end class_

//@}

//----------------------------------------------------------------------------
// GLOBAL_ANTI_TARGET_CRITERION
//----------------------------------------------------------------------------
/**@name Le critère d'atteinte des anticibles */
//@{
          ///
class GlobalAntiTargetCriterion : public Criterion 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  GlobalAntiTargetCriterion();  // _no swig_
//@}

protected :

}; // _end class_

//@}

//----------------------------------------------------------------------------
// UTILITY_CRITERION
//----------------------------------------------------------------------------
/**@name Les critères d'utilité d'une activité */
//@{

/**@name La classe de base */
//@{
          ///
class UtilityCriterion : public GoalCriterion 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  UtilityCriterion();  // _no swig_
           /// 
float UtilityByActivityList();
      
//@}

protected :

}; // _end class_

//@}

//----------------------------------------------------------------------------
// TARGET_UTILITY_CRITERION
//----------------------------------------------------------------------------

/**@name Utilité vis-à-vis des cibles */
//@{
          ///
class TargetUtilityCriterion : public UtilityCriterion 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  TargetUtilityCriterion();  // _no swig_
      
//@}

protected :

}; // _end class_
//@}

float targetUtilityCriterion_getCriterionValue_Body(EntityMethod*);

//----------------------------------------------------------------------------
// ANTI_TARGET_UTILITY_CRITERION
//----------------------------------------------------------------------------

/**@name Utilité vis-à-vis des anticibles */
//@{
          ///
class AntiTargetUtilityCriterion : public UtilityCriterion 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  AntiTargetUtilityCriterion();  // _no swig_
      
//@}

protected :

}; // _end class_
//@}

float antiTargetUtilityCriterion_getCriterionValue_Body(EntityMethod*);

//@}  // fin critères d'utilité

//@}  // fin critères prédéfinis

//@} // fin Les critères de décision (ouverture dans C_Criterion)



#endif  // _C_CORECRITERION_HH

