//============================================================================
// FILE: C_Goal.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des buts
// 
// AUTHORS: Slougui (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_GOAL_HH
#define _C_GOAL_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"

//----------------------------------------------------------------------------
// GOAL
//----------------------------------------------------------------------------
/**@name Les buts
\label{sectionGoal}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Goal.h} \end{itemize}
*/

//@{   // buts + cibles + anticibles

/**@name Les classes des buts en général */
//@{

/**@name La classe de base */
//@{
          ///
class Goal : public Entity 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  Goal();  // _no swig_
           ///
  //float ValueSatisfaction(); // obsolète ?
      
//@}

protected :

}; // _end class_

//@}

/**@name Les buts à valeur entière */
//@{

/**@name La classe */
//@{
           ///
class IntGoal : public Goal
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  IntGoal();  // _no swig_
           ///
int CurrentValue();
           ///
float SatisfactionByFutureValue(Entity*);
           ///
float SatisfactionByCurrentValue();
      
//@}

protected :

}; // _end class_

//@}

/**@name Valuation entière du but par un jeu d'activité */
//@{
           /// ... comme la moyenne des valeurs sur les activités.
int meanStandardIntGoal_intFutureValueByActivityList_Body(EntityMethod*);
           /// ... comme la plus petite valeur.
int minStandardIntGoal_intFutureValueByActivityList_Body(EntityMethod*);
           /// ... comme la plus grande valeur.
int maxStandardIntGoal_intFutureValueByActivityList_Body(EntityMethod*);
//@}

//@} // fin buts entiers

/**@name Les buts à valeur flottante.
*/
//@{

/**@name La classe
*/
//@{
           ///
class FloatGoal : public Goal 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  FloatGoal();  // _no swig_
           ///
float CurrentValue();
           ///
float SatisfactionByFutureValue(Entity*);
           ///
float SatisfactionByCurrentValue();
      
//@}

protected :

}; // _end class_
      
//@}

/**@name Valuation réelle du but par un jeu d'activité */
//@{
           /// ... comme la moyenne des valeurs sur les activités.
float meanStandardFloatGoal_floatFutureValueByActivityList_Body(EntityMethod*);
           /// ... comme la plus petite valeur.
float minStandardFloatGoal_floatFutureValueByActivityList_Body(EntityMethod*);
           /// ... comme la plus grande valeur.
float maxStandardFloatGoal_floatFutureValueByActivityList_Body(EntityMethod*);
      
//@}
      
//@} // fin buts flottants

//@} // fin buts en général


//----------------------------------------------------------------------------
// GOAL_TARGET
//----------------------------------------------------------------------------

/**@name Des buts particuliers : les cibles
Une cible est ...
*/
//@{

/**@name Cibles à valeur entière */
//@{
          ///
class IntTarget : public IntGoal 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  IntTarget();  // _no swig_
      
//@}

protected :

}; // _end class_ 
 
//@}

/**@name Cibles à valeur flottante */
//@{ 
          ///
class FloatTarget : public FloatGoal 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  FloatTarget();  // _no swig_
      
//@}

protected :

}; // _end class_
 
//@}

//@} // fin cibles

//----------------------------------------------------------------------------
// GOAL_ANTI_TARGET
//----------------------------------------------------------------------------

/**@name D'autres buts particuliers : les anticibles
 Une anticible est ...
*/
//@{

/**@name Anticibles à valeur entière */
//@{
           ///
class IntAntiTarget : public IntGoal 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  IntAntiTarget();  // _no swig_
      
//@}

protected :

}; // _end class_

//@}

/**@name Anticibles à valeur flottante */
//@{
           ///
class FloatAntiTarget : public FloatGoal 
{
public:

/**@name LIFECYCLE */
//@{ 
           /// Constructeur de base.
  FloatAntiTarget();  // _no swig_
      
//@}

protected :

}; // _end class_

//@}

//@} // fin anticibles
//----------------------------------------------------------------------------


//@} // fin buts + cibles + anticibles


#endif  // _C_GOAL_HH
