//============================================================================
// FILE: C_ResourceAllocation.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des classes représentant des jeux de ressources allouées
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_RESOURCEALLOCATION_HH
#define _C_RESOURCEALLOCATION_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_Operation.h"

// EXTERNAL REFERENCES

/////////////////////////////////////////////////////////////////////////////
//
// INSTRUCTION_ALLOCATION
//
/////////////////////////////////////////////////////////////////////////////

/**@name Structures pour les allocations de ressources
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_ResourceAllocation.h} \end{itemize}
*/ 
//@{

/**@name Jeu de ressources allouées à une activité primitive
Structure V de l'ontologie.
*/ 
//@{
          ///
class InstructionAllocation : public BasicEntity
{
public:
          // Constructeur de base.
  InstructionAllocation();
          // Constructeur par copie.
  InstructionAllocation(const InstructionAllocation&);
          // Destructeur.
  ~InstructionAllocation();
          // 
  void        DeleteInDepth();

          /// 
  pEntityTab* ExtractRscSet();
          /// 
  pEntityTab* ExtractPerfSet();
          /// 
  pEntityTab* ExtractOOSet();  // + 110217
          /// 
  pEntityTab* ExtractOtRscSet();  // + 110217
          /// 
  pEntityTab* ExtractRscSetButOperation();
          // 
  void        DisplayRscSet();
          // 
  Boolean IsBetterAllocatedThan(InstructionAllocation*, Entity*, Boolean);

 };

//@}  // fin InstructionAllocation

/////////////////////////////////////////////////////////////////////////////
//
// INSTRUCTION_SET_ALLOCATION
//
/////////////////////////////////////////////////////////////////////////////

/**@name CoengagementItem ...
*/ 
//@{
          ///
class CoengagementItem : public DescriptedEntity
{
public:
          // Constructeur de base.
  CoengagementItem();
          // Constructeur par copie.
  CoengagementItem(const CoengagementItem&);
  /*
          // Destructeur.
  ~CoengagementItem();
  */
 };

//@}  // fin CoengagementItem

//===========================================================================

/**@name UsageItem ...
*/ 
//@{
          ///
class UsageItem : public DescriptedEntity
{
public:
          // Constructeur de base.
  UsageItem();
          // Constructeur par copie.
  UsageItem(const UsageItem&);
  /*
          // Destructeur.
  ~UsageItem();
  */
          // 
  void        DeleteInDepth();

          /// Renvoie le {\bf GetCoengagementItem} portant sur la ressource ou l'opération dont le symbole est passé en paramètre.\index{méthodes!GetCoengagementItem}
  CoengagementItem* GetCoengagementItem(UserClassId);

          /// Renvoie la liste des symboles de classe qui font l'objet d'un {\bf CoengagementItem} dans ce {\bf UsageItem}.\index{méthodes!GetCoengagementItemIdList}
 TableauInt*        GetCoengagementItemIdList();

 };

//@}  // fin UsageItem

//---------------------------------------------------------------------------

/**@name ResourceUsageItem ...
*/ 
//@{
          ///
class ResourceUsageItem : public UsageItem
{
public:
          // Constructeur de base.
  ResourceUsageItem();
          // Constructeur par copie.
  ResourceUsageItem(const ResourceUsageItem&);
          // Destructeur.
  ~ResourceUsageItem();
 };

//@}  // fin PureResourceUsageItem

//---------------------------------------------------------------------------

/**@name OperationUsageItem ...
*/ 
//@{
          ///
class OperationUsageItem : public UsageItem
{
public:
          // Constructeur de base.
  OperationUsageItem();
          // Constructeur par copie.
  OperationUsageItem(const OperationUsageItem&);
          // Destructeur.
  ~OperationUsageItem();
 };

//@}  // fin OperationUsageItem

//---------------------------------------------------------------------------

/**@name Jeu de ressources allouées à un jeu d'activités
Structure U de l'ontologie.
*/ 
//@{
          ///
class InstructionSetAllocation : public BasicEntity
{
public:
          // Constructeur de base.
  InstructionSetAllocation();
  InstructionSetAllocation(int);  // 100312 to scratch
  InstructionSetAllocation(int, int);  // 100312 to scratch
          // Constructeur par copie.
  InstructionSetAllocation(const InstructionSetAllocation&);
          // Destructeur.
  ~InstructionSetAllocation();
          // 
  void        DeleteInDepth();

  /// Extrait la liste de toutes les structures {\bf CoengagementItem} qui se trouvent dans les différents {\bf UsageItem} de la présente allocation. Il n'y a qu'un seul {\bf CoengagementItem} portant sur une resource ou une opération donnée {\em obj}. On y rassemble toutes les ressources et l'opération qu'on trouve coengagées avec {\em obj} dans les différents {\bf UsageItem}.\index{méthodes!ExtractCoengItemGlobalList}
  pEntityTab* ExtractCoengItemGlobalList();

          /// Extrait la liste d'entités (opération ou ressource) d'une structure {\bf InstructionSetAllocation} (allocation de resources à un jeu d'activités primitives).\index{méthodes!ExtractRscSet}
  pEntityTab* ExtractRscSet();

          /// Extrait une structure {\bf InstructionAllocation} (allocation de resources à une activité primitive {\em act}) d'une structure {\bf InstructionSetAllocation} (allocation de ressources à un jeu d'activités primitives), mais supposée construite sur un jeu avec la seule activité {\em act}.\index{méthodes!ExtractVAlloc}
  InstructionAllocation* ExtractVAlloc();

          // Extrait l'{\bf Operation} {\em ot} désignée dans une structure {\bf InstructionSetAllocation} (allocation de resources à un jeu d'activités primitives), mais supposée construite sur un jeu avec la seule activité {\em act} sur laquelle pourrait porter {\em ot}.\index{méthodes!ExtractOperation}
  Operation* ExtractOperation();

          /// Augmente une structure U (cf. ontologie) avec l'ensemble de resources qui sont les éléments de l'entité passée en paramètre.\index{méthodes!ReturnExtensionWithComb}
    //InstructionSetAllocation* ReturnExtensionWithComb(AllocatedSet*);
    InstructionSetAllocation* ReturnExtensionWithComb(Entity*); 

          /// Augmente cette structure U (cf. ontologie) par une autre. La première est physiquement modifiée. \index{méthodes!ExtendItWithAlloc}
  void ExtendItWithAlloc(InstructionSetAllocation*);

          // Crée une copie de cette structure U (cf. ontologie) et retourne la copie augmentée par une autre. La première n'est pas physiquement modifiée. \index{méthodes!ReturnExtensionWithAlloc}
  //InstructionSetAllocation* ReturnExtensionWithAlloc(const InstructionSetAllocation*);
  InstructionSetAllocation* ReturnExtensionWithAlloc(InstructionSetAllocation*);

          // Renvoie le {\bf UsageItem} portant sur la ressource ou l'opération passée en argument.\index{méthodes!GetUsageItem}
  UsageItem* GetUsageItem(Entity* pObj);

          // \index{méthodes!CheckActivityInconsistencyConditions}
  Boolean CheckActivityInconsistencyConditions(PrimitiveActivity* pAct);

          // \index{méthodes!CheckActivityInconsistencyConditions}
  Boolean CheckResourceSharingViolationConditions(PrimitiveActivity* pAct);
          ///
  void               SpecialDisplay();
 };

//@}  // fin InstructionSetAllocation

//===========================================================================


//===========================================================================



//@}  // fin Allocations de ressources

#endif  // _C_RESOURCEALLOCATION_HH
