//============================================================================
// FILE: O_CoreTypes.h 
//
// TYPE: C++
// DESCRIPTION: définition des types "moteur" et des variables globales
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================#

#ifndef _O_CORETYPES_H
#define _O_CORETYPES_H

// SYSTEM INCLUDES
//
//#include <iostream.h>

// PROJECT INCLUDES
//

class InterfaceDataItem;

#include "B_diese.h"

extern int (*f)();

// EXTERN_C void boot_bdiese _((CV* cv)); // à tester (031016)

//typedef Tableau<Activity*>              pActivityTab;

/**@name D\'efinitions globales et g\'en\'erales
\label{classeOdCoreTypes}
\begin{itemize} \item Directive d'inclusion : {\tt \#include O_CoreTypes.h} \end{itemize} 
*/
//@{

/**@name Variables globales
 */
//@{
         // tableaux de conversion enum -> chaine de caractères
// extern TableauString StateIdToCharTab;   // + JPR 0510
         // fonctions de conversion associées
         // invoquées dans O_EnumToInt, 
         // laquelle surcharge EnumToInt dans C_ParserSimu.yacc.y
         // initialisée à D_EnumToInt dans ParserSimu.yacc.y
 // void C_MakeStateIdToCharTab();
 // extern TableauString ContinuationModeToCharTab;  

/**@name Variables globales pour la taille des énumérations 
Indiquent le plus grand numéro symbolique dans les énumérations {\bf UserClassId}, {\bf UserDescriptorId} \ldots . Les numéros symboliques dans les énumérations complémentaires dans une sous-couche immédiate de \OD doivent être supérieures à ces valeurs. Ces valeurs doivent être modifiées avec les plus grandes valeurs attribuées dans la sous-couche. Ces règles doivent être à nouveau appliquées pour une sous-couche encore inférieure.\\

extern int g_BD_ReservedUserClassId      =302;\\
extern int g_BD_ReservedUserDescriptorId =302;\\
extern int g_BD_ReservedUserMethodId     =202;\\
extern int g_BD_ReservedUserArgumentId   =102;\\
extern int g_BD_ReservedUserProcessId    =  8;\\
extern int g_BD_ReservedUserEventId      = 16;\\
extern int g_BD_ReservedUserFileId       = -9;\\
*/ 
// @{
    // intentionnaly blank

enum O_TimeReferenceOption {
  TM_UTC=0, TM_LOCAL=1
};

extern int gTimeReferenceOption;


// @}  // fin var globales taille enum


//@}  // fin  Variables globales
extern int g_OD_constCount;
extern int g_OD_destCount;

/* *@name Constantes */
// @{

//----------------------------------------------------------------------------
//      Symboles des classes d'événements
//----------------------------------------------------------------------------

/* *@name O_EventClassId : événements prédéfinis
\noindent \index{O_EventClassId (énumération)}\\
UPDATE\_SITUATION\_EVENT=21, MAKE\_INSTRUCTION\_LIST\_EVENT
Enumération exhaustive des symboles de classe des événements prédéfinis.\index{événements!symboles de classe}
*/
//----------------------------------------------------------------------------
// cf. g_D_ReservedUserEventId  ; mettre à jour g_OD_ReservedUserEventId
// enum O_EventClassId {UPDATE_SITUATION_EVENT=21, MAKE_INSTRUCTION_LIST_EVENT
//};

//----------------------------------------------------------------------------
//      Symboles des classes de processus
//----------------------------------------------------------------------------

/* *@name O_ProcessClassId : processus prédéfinis
\noindent \index{O_ProcessClassId (énumération)}
CHECK\_TRAJECTORY=11, UPDATE\_SITUATION\_PROCESS

Enumération exhaustive des symboles de classe des processus prédéfinis.\index{processus!symboles de classe}
*/
//----------------------------------------------------------------------------
// cf. g_D_ReservedUserProcessId  ; mettre à jour g_OD_ReservedUserProcessId
// enum O_ProcessClassId {CHECK_TRAJECTORY=11, UPDATE_SITUATION_PROCESS
//};


//----------------------------------------------------------------------------
//      Symboles des classes d'entités
//----------------------------------------------------------------------------

/* *@name O_EntityClassId : entités prédéfinies

Attention ! les entiers correspondants aux sous-classes d'entités doivent être tous différents, qu'il s'agisse de particularisations d'activités, d'opérations ou d'une autre sous-classe d'entités. \\

Les entiers 0, 1 et 2 sont réservés aux classes d'entités de DIESE.\\
{\bf Activités :} \\

INTERFACE_DATA_ITEM=301, 
//PRIMITIVE\_ACTIVITY
 */

// cf. g_D_ReservedUserClassId  ; mettre à jour g_OD_ReservedUserClassId
enum O_EntityClassId {
INTERFACE_DATA_ITEM=301 
//PRIMITIVE_ACTIVITY
/* LAST=302*/
/* ne pas oublier d'ajuster O_MakeDescIdToCharTab dans O_CoreFunction.cc */
/* ne pas oublier d'ajuster O_InstallMetaClasses dans C_CoreFunction.cc */
};

//----------------------------------------------------------------------------
//      Symboles des Descripteurs 
//----------------------------------------------------------------------------

/* *@name O_DescriptorId : descripteurs prédéfinis
Constants :\\

Variables :\\

Enumération exhaustive des descripteurs gérés par le moteur de simulation. \index{O\_DescriptorId}
*/

//cf g_D_ReservedUserDescriptorId ; mettre à jour g_OD_ReservedUserDescriptorId
enum O_DescriptorId {
  // constants
  DATE_ISO_ATTRIBUTE=301,
ADD_VALUE_MODE=302
//UN_AUTRE_ATTRIBUTE,                     // 302
/* ne pas oublier d'ajuster O_MakeDescIdToCharTab dans O_CoreFunction.cc */
  // variables
//ACHIEVEMENT_DEGREE=351,
//ACTIVITY_ACTION_MODE                     // 352
/* LAST=352*/
/* ne pas oublier d'ajuster O_MakeDescIdToCharTab dans O_CoreFunction.cc */
/* ne pas oublier d'ajuster O_InstallMetaClasses dans C_CoreFunction.cc */
};


//----------------------------------------------------------------------------
//      Symboles des Méthodes
//----------------------------------------------------------------------------

/* *@name O_CoreMethodId : méthodes prédéfinies

OPENING\_PREDICATE=201, CLOSING\_PREDICATE

Enumération exhaustive des méthodes gérées par le moteur de simulation.
*/
// cf. g_D_ReservedUserMethodId ; mettre à jour g_OD_ReservedUserMethodId
// !!!! METTRE À JOUR O_MakeMethodIdToCharTab !!!!
enum O_CoreMethodId {
//OPENING_PREDICATE=201, CLOSING_PREDICATE
/* LAST=202*/
};


//----------------------------------------------------------------------------
//      Symboles des Arguments
//----------------------------------------------------------------------------

/* *@name O_CoreArgumentId : arguments prédéfinis

OPERATION\_TARGET=101, TRANSFER\_OPERATION\_SOURCE\\

Enumération exhaustive des arguments de méthodes, gérés par le moteur de simulation.
*/
// cf. g_D_ReservedUserArgumentId ; mettre à jour g_OD_ReservedUserArgumentId
enum O_CoreArgumentId {
//OPERATION_TARGET=101, TRANSFER_OPERATION_SOURCE
/* LAST=102*/
};
// @}  // fin constantes


//@} // fin Définitions globales et générales

#endif  // _O_CORETYPES_H
