//============================================================================
// FILE: _O_Simulation.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la spécialisation de la classe des simulations
//              avec ajouts de code pour la connexion a un client externe
//              tel que OpenFluid
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _O_SIMULATION_HH
#define _O_SIMULATION_HH

// SYSTEM INCLUDES
//
//#include <string>  // 080319
////using std::string;

// PROJECT INCLUDES
//
#include "Simulation.h"

/**@name CDPFORTEXLa classe des simulations serveur d'un client externe
\begin{itemize} \item Directive d'inclusion : {\tt \#include O_Simulation.h} \end{itemize}
 
La classe {\bf OpenSimulation} est une spécialisation de la classe {\bf Simulation}. Alors que le message adressé à une instance de {\bf Simulation} était {\bf Run}, celui adressé à une instance de {\bf OpenSimulation} est {\bf RunOpenSimulation}.\\

Elle doit être exploitée lorsque la fin de la période de simulation confiée à un et un seul appel à la méthode {\bf Run} peut correspondre, non plus seulement comme dans les couches \BD et \CD à :
\begin{itemize}
\item l'atteinte de la date de fin de simulation, définie par le message {\bf SetEndClock} ou {\bf SetEndDate} adressé à la simulation en cours ;
\item le point où l'agenda d'événements est devenu vide ;
\item le point où le message {\bf Stop} a été adressé à la simulation en cours;
\end{itemize}
... mais aussi :
\begin{itemize}
\item une date spécifiée sous la forme d'un argument complémentaire à la méthode {\bf Run}, normalement antérieure à la date de fin nominale.
\end{itemize}

Il est cependant possible, après avoir instancié la classe {\bf OpenSimulation}, de lui adresser un message {\bf RunOpenSimulation} qui n'indique pas de date d'arrêt particulière. Dans ce cas, le moteur de simulation se comporte comme si on avait instancié la classe {\bf Simulation}.\\

Lorsqu'une date d'arrêt particulière est à indiquer, on la place en argument du message {\bf RunOpenSimulation}, dans le format ISO 8601 "aaaa-mm-qqThh:mi:sszzzzzz" (par exemple "2005-02-15T00:00:00+00:00").\\

Un cas d'utilisation typique est l'enchaînement d'une série d'appel à {\bf RunOpenSimulation} qui partitionnent la simulation globale en une séquence d'épisodes de simulation \index{épisode de simulation}\label{episodeSimulation}entre lesquels s'opère une communication entre le programme-maître et le simulateur.\\

A cette fin de communication, deux arguments complémentaires de ma méthode {\bf RunOpenSimulation} permettent au programme maître de transmettre et de recevoir des informations, vers et en provenance du simulateur. Les valeurs de ces deux arguments sont des tableaux, respectivement 'in' et 'out', d'instances de la classe {\bf InterfaceDataItem}. Voir la documentation de cette classe.\\

Avant l'invocation de cette méthode par une procédure cliente, ce client a valué, pour chaque instance de {\bf InterfaceDataItem} du tableau 'in', ses descripteurs qui portent (i) les identifications de l'entité visée et de son descripteur visé, (ii) la valeur de ce descripteur et (iii) la date à laquelle elle a été capturée ou pour laquelle elle est valide. Si la date n'est pas précisée, c'est la valeur à l'instant de l'invocation qui est supposée transmise par le client. Noter que ces valeurs seront affectées aux descripteurs avant toute autre tâche, procédure ou opération à cette date-là. On peut donc les considérer comme des valeurs acquises au pas précédent.\\   

Le client a aussi valué pour les instances du tableau 'out' les descripteurs portant les identifications de l'entité visée, de son descripteur visé et de la date à laquelle la valeur doit être capturée, aprè toute autre tâche, procédure ou opération à cette date-là. Si la date n'est pas précisée, c'est la valeur à l'instant 'pauseDate' qui sera transmise au client. Avant le retour à la procédure cliente, la présente méthode écrit dans chaque instance la valeur du descripteur visé.
*/ 
//@{
        ///
class OpenSimulation : public Simulation
{
public:

  /**@name LIFECYCLE */
//@{
          ///Constructeur de base. On fait implicitement appel au constructeur de la classe de base {\bf Simulation}.
  OpenSimulation(); // _no swig_    
//@}                         

/**@name OPERATIONS */
//@{ 
          /// Exécute la simulation, en dépilant l'agenda d'événements jusqu'à ce qu'il soit vide, ou bien qu'un certain nombre de dépilements d'événements, donné en argument, aient eu lieu, ou bien que la date de fin de simulation soit dépassée.\index{méthodes!RunOpenSimulation}
   void            RunOpenSimulation(int);  // _no swig_  \label{defRun}

          /// Exécute la simulation, en dépilant l'agenda d'événements jusqu'à ce qu'il soit vide, ou bien jusqu'à ce que la date de fin de simulation soit dépassée, et tant qu'un message {\bf Stop} n'a pas été adressé à la simulation en cours.
   void            RunOpenSimulation(); // _no swig_ 

         /// Exécute la simulation, en dépilant l'agenda d'événements jusqu'à ce que la date passée en second argument soit dépassée (dans ce cas, la valeur renvoyée est FALSE) ou bien jusqu'à ce que l'agenda soit vide, ou bien jusqu'à ce que la date de fin de simulation soit dépassée (dans ces deux derniers cas, la valeur renvoyée est TRUE). La valeur renvoyée indique donc au programme appelant s'il y a lieu (valeur FALSE) ou non (valeur TRUE) d'invoquer à nouveau la présente méthode. L'argument 'iterationCounter', généralement initialisé à 0 (zéro) par le programme appelant, est incrémenté de 1 par la présente méthode lors de chaque dépilement d'événement. \index{méthodes!RunOpenSimulationToDate}  
   int             RunOpenSimulationToDate(int& iterationCounter,
										   char* pauseDate); // _no swig_

         /// Exécute la simulation comme la méthode ci-dessus, avec la même sémantique pour la valeur renvoyée. Le tableaux 'in' assure le passage d'information du client externe vers la simulation DIESE. Le tableaux 'out' assure le passage d'information de la simulation DIESE vers le client externe. Leurs éléments sont des instances de la classe {\bf InterfaceDataItem}. Ces instances véhiculent la valeur d'un descripteur d'une entité, capturée à une certaine date, ou valide à partir d'une certaine date. 
   int             RunOpenSimulationToDate(int& iterationCounter,
										   pEntityTab* in,
										   char* pauseDate,
										   pEntityTab* out); // _no swig_

         /// Exécute la simulation (avec la même sémantique pour les tableaux 'in' et 'out') jusqu'à une certaine date, et renvoie cette date au client externe. Pour chaque élément dans le tableau 'in', seulement s'il se rapporte à une date antérieure à la date retournée, le passage d'information du client externe vers la simulation DIESE est effective. Les éléments dans le tableau 'out' se rapportent nécessairement à une date non postérieure à la date retournée.
   char*             RunOpenSimulation(int& iterationCounter,
									   pEntityTab* in,
									   pEntityTab* out); // _no swig_
//@}
};


//@} // fin classe Simulation

//=X===============X=============================X============================



#endif  // _O_SIMULATION_HH
